@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2006-2007 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2006-2008 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -206,8 +206,13 @@ read_command()
   debug_printf(DEBUG_NOTE, "read command\n");
 
   do {
-	len = read(STDIN_FILENO, rbuf, sizeof(rbuf) - 1);
-	rbuf[len] = '\0';
+	if ((len = read(STDIN_FILENO, rbuf, sizeof(rbuf) - 1)) == -1 || len == 0) {
+	  rbuf[0] = '\n';
+	  rbuf[1] = '\0';
+	  len = 1;
+	  debug_printf(DEBUG_NOTE, "stdin has corrupted\n");
+	} else
+	  rbuf[len] = '\0';
 
 	if (strlen(cmdbuf) + len + 1 > cmdbuf_len) {
 	  cmdbuf_len += DEFAULT_MESSAGE_SIZE;
