@@ -1,5 +1,5 @@
 ;; 
-;;  Copyright (c) 2005-2007 uim Project http://uim.freedesktop.org/
+;;  Copyright (c) 2005-2008 uim Project http://code.google.com/p/uim/
 ;;
 ;;  All rights reserved.
 ;;
@@ -77,21 +77,28 @@
 ;;
 ;; Get original key-mapped function
 ;;
-(defun uim-getbind (keyvec)
+(defun uim-key-binding (vec &optional translation)
   (let (bind 
 	(mode uim-mode))
     (unwind-protect
 	(progn
 	  (setq uim-mode nil)
+	  (setq bind (key-binding vec))
 
-	  (setq bind (key-binding (vector (aref keyvec 0))))
+	  (if (and (or (not bind)
+		       (integerp bind))
+		   translation)
+	      (setq bind (lookup-key key-translation-map vec))))
 
-	  (if (not (memq bind (list 'universal-argument 
-				    'digit-argument 
-				    'negative-argument)))
-	      (setq bind (key-binding keyvec)))
-	  )
       (setq uim-mode mode))
+
+    ;; process autoload keymap
+    (if (and (symbolp bind) (commandp bind))
+	(let ((sf (symbol-function bind)))
+	  (if (and (listp sf)
+		   (eq 'keymap (nth 4 sf)))
+	      (lookup-key bind [nil]))))
+
     bind
     ))
 
@@ -136,13 +143,104 @@
   )
 
 
-(defun uim-command-execute (keyvec)
-  (let ((mode uim-mode))
+(defun uim-this-command-keys-override ()
+  (if (not uim-this-command-keys-original)
+      (let ((doc (documentation 'this-command-keys)))
+	(setq uim-this-command-keys-original
+	      (symbol-function 'this-command-keys))
+	(eval
+	 `(fset 'this-command-keys 
+		'(lambda ()
+		   ,doc
+		   (if (and (boundp 'uim-key-vector) uim-key-vector)
+		       uim-key-vector
+		     (funcall uim-this-command-keys-original))))))))
+
+
+(defun uim-this-command-keys-restore ()
+  (if uim-this-command-keys-original
+      (progn
+	(fset 'this-command-keys uim-this-command-keys-original)
+	(setq uim-this-command-keys-original nil))))
+
+
+;; for Emacs21
+(defun uim-read-char-exclusive-override ()
+  (if (not uim-read-char-exclusive-original)
+      (let ((doc (documentation 'read-char-exclusive)))
+	(setq uim-read-char-exclusive-original
+	      (symbol-function 'read-char-exclusive))
+	(eval
+	 `(fset 'read-char-exclusive
+		'(lambda (&optional prompt inherit-input-method)
+		   ,doc
+		   (if (and (boundp 'uim-key-vector) uim-key-vector)
+		       (let (executing-macro)
+			 (funcall uim-read-char-exclusive-original 
+				  prompt inherit-input-method))
+		     (funcall uim-read-char-exclusive-original))))))))
+
+
+;; for Emacs21
+(defun uim-read-char-exclusive-restore ()
+  (if uim-read-char-exclusive-original
+      (progn
+	(fset 'read-char-exclusive uim-read-char-exclusive-original)
+	(setq uim-read-char-exclusive-original nil))))
+
+
+(defun uim-command-execute (uim-key-vector &optional bind)
+
+  (let (map
+	(buffer (current-buffer)))
+
     (unwind-protect
 	(progn
-	  (setq uim-mode nil)
-	  (command-execute keyvec))
-      (setq uim-mode mode))))
+
+	  (if (and bind
+		   (commandp bind))
+	      (setq this-command bind)
+	    (setq this-command uim-key-vector))
+
+
+	  (when uim-xemacs
+	    (setq last-input-event uim-original-input-event)
+	    (handle-pre-motion-command))
+
+	  (run-hooks 'pre-command-hook)
+
+	  (setq last-command-char 
+		(aref (uim-get-vector-from-tail uim-key-vector 1) 0))
+	  
+	  (setq map (cdr (assq 'uim-mode minor-mode-map-alist)))
+	  (setcdr (assq 'uim-mode minor-mode-map-alist) uim-dummy-map)
+
+	  (if (or (and bind
+		       (eq bind 'digit-argument))	  
+		  (and uim-xemacs
+		       (not (eventp last-command-char))))
+	      (command-execute uim-key-vector)
+	    (command-execute this-command))
+
+	  (setq last-command this-command)
+	  ;;(setq last-command-char (aref uim-key-vector 0))
+
+	  (if uim-xemacs
+	      (handle-post-motion-command))
+
+	  (if (eq bind 'self-insert-command)
+	      (uim-concat-undo)
+	    (uim-flush-concat-undo)))
+
+      (if (and buffer 
+	       (buffer-live-p buffer))
+	  (progn 
+	    (set-buffer buffer)
+	    (setcdr (assq 'uim-mode minor-mode-map-alist) map)
+	    ))
+      )))
+
+
 
 
 (defun uim-blink-match (char)
@@ -156,13 +254,13 @@
 (defun uim-process-mouse-event (event)
   (cond (uim-emacs
 	 (let* ((mouse-event (car event))
-		(bind (uim-getbind (vector mouse-event))))
+		(bind (uim-key-binding (vector mouse-event))))
 	   (if (commandp bind)
 	       (call-interactively bind nil (vector event))
 	     (if (not (memq 'down (event-modifiers mouse-event)))
 		 (undefined)))))
 	(uim-xemacs
-	 (let* ((bind (uim-getbind (vector event))))
+	 (let* ((bind (uim-key-binding (vector event))))
 	   (if (commandp bind)
 	       (call-interactively bind nil (vector event))))))
   )
@@ -171,74 +269,69 @@
 ;;
 ;; Process the key vector returned from Uim.
 ;; 
-(defun uim-process-keyvec (keyvec &optional count)
-  (let ((bind (uim-getbind keyvec))
-	keyvectmp)
+(defun uim-process-key-vector (key-vector &optional count)
 
+  (let ((bind (uim-key-binding key-vector t))
+	undef
+	vtmp
+	continue
+	)
+    
     (if uim-emacs
 	;; for transient-mark-mode
 	(setq deactivate-mark nil))
-    
-    (unwind-protect    
-	(cond ((and (or (keymapp bind) (not bind))
-		    (setq keyvectmp (uim-remove-shift keyvec))
-		    (setq keyvec keyvectmp))
 
-	       (if uim-xemacs
-		   (progn
-		     (setq uim-retry-keys keyvec)
-		     (if count (setq prefix-arg count))
-		     (setq unread-command-events
-			   (cons (aref keyvec 0) unread-command-events))))
+    (unwind-protect
+	(cond ((keymapp bind)
+	       (setq continue t)
+	       )
+
+	      ((stringp bind)
+	       (if count
+		   (setq prefix-arg count))
+	       (uim-command-execute key-vector bind))
 
-	       (if uim-emacs
-		   (progn
-		     (uim-debug (format "retry key is %s / count is %s" keyvec
-					count))
-		     (setq uim-retry-keys keyvec)
-		     (if count (setq prefix-arg count))
-		     ;; dummy data
-		     (setq unread-command-events
-			   (nconc (listify-key-sequence keyvec)
-				  unread-command-events)
-			   )
-		     ))
-	       )
-	      (count
-	       (setq prefix-arg count)
-	       (uim-command-execute
-		(uim-getbind keyvec))
-		;;(uim-getbind (uim-last-onestroke-key keyvec)))
-	       )
-	      
 	      ((commandp bind)
-	       (if (eq bind 'self-insert-command)
-		   (progn
-		     (setq this-command bind)
-		     (setq last-command-char (aref keyvec 0))
-		     (call-interactively bind)
-		     (uim-concat-undo))
-		 (setq this-command bind)
-		 (command-execute bind) 
-		 (uim-flush-concat-undo)
-		 ))
+	       (if count
+		   (setq prefix-arg count))
+	       (uim-command-execute key-vector bind))
+
+	      ((or (and uim-emacs
+			(setq vtmp (aref (uim-last-onestroke-key key-vector) 0))
+			(integerp vtmp)
+			(equal help-char vtmp))
+		   (and uim-xemacs
+			(setq vtmp (aref (uim-last-onestroke-key key-vector) 0))
+			(equal (uim-xemacs-make-event 
+				(uim-convert-char-to-symbolvector 
+				 (key-description help-char)))
+			       vtmp)))
+	       (uim-command-execute key-vector))
+
 	      (t
-	       (uim-flush-concat-undo)
-	       (if uim-xemacs
-		   (error 'undefined-keystroke-sequence 
-			  (uim-xemacs-make-event 
-			   (uim-convert-char-to-symbolvector 
-			    (key-description keyvec))))
-		 (undefined))
-	       )
-	      )
+	       (setq undef key-vector)))
+      
+      (when undef
+	(uim-flush-concat-undo)
+	(if uim-xemacs
+	    (error 'undefined-keystroke-sequence 
+		   (uim-xemacs-make-event 
+		    (uim-convert-char-to-symbolvector 
+		     (key-description key-vector))))
+	  )
+	(when uim-emacs
+	  (undefined)
+	  (setq uim-keystroke-displaying nil)
+	  (if (>= emacs-major-version 22)
+	      (let (message-log-max)
+		(message "%s is undefined" (key-description undef)))
+	  )))
+
       (if uim-emacs
 	  (setq uim-deactivate-mark deactivate-mark))
       )
-    )
-  )
-  
-
+    continue
+    ))
 
 
 (defun uim-xemacs-restore-menubar ()
@@ -352,35 +445,43 @@
 
 
 
+(defun uim-check-shift (input-vector)
+  (eval (cons 'or
+	      (mapcar
+	       '(lambda (x)
+		  (or (and uim-emacs
+			   (or (and (integerp x) (/= (logand (lsh 1 25) x) 0))
+			       (string-match "S-" (format "%s" x))))
+		      (and uim-xemacs
+			   (string-match "Sh-" 
+					 (key-description input-vector)))))
+	       (append input-vector nil)))))
+
 
 ;;
 ;; remove Shift modifier from key vector
 ;;
-(defun uim-remove-shift (keyvec)
-  (cond (uim-emacs
-	 (let ((keyval (aref keyvec 0)))
-	   (if (and (integerp keyval)
-		    (/= (logand (lsh 1 25) keyval) 0))
-	       (setq keyvec (vector (logand (lognot (lsh 1 25)) keyval)))
-	     (let ((keystr (format "%s" keyval)))
-	       (if (string-match "S-" keystr)
-		   (setq keyvec 
-			 (vector (read 
-				  (replace-match "" nil nil keystr))))
-		 (setq keyvec nil))))))
-	(uim-xemacs
-	 (let ((keystr (key-description keyvec)))
-	   (if (string-match "Sh-" keystr)
-	       (setq keyvec 
-		     (vector (uim-xemacs-make-event 
-			      (uim-convert-char-to-symbolvector 
-			       (replace-match "" nil nil keystr)))))
-	     (setq keyvec nil))))
-	(t 
-	 (setq keyvec nil))
-	)
-  keyvec
-  )
+(defun uim-remove-shift (input-vector)
+  (vconcat (mapcar 
+	   '(lambda (x)
+	      (cond (uim-emacs
+		     (if (and (integerp x)
+			      (/= (logand (lsh 1 25) x) 0))
+			 (logand (lognot (lsh 1 25)) x)
+		       (let ((key-str (format "%s" x)))
+			 (if (string-match "S-" key-str)
+			     (read (replace-match "" nil nil 
+						  key-str))
+			   x))))
+		    (uim-xemacs
+		     (let ((key-str 
+			    (key-description input-vector)))
+		       (if (string-match "Sh-" key-str)
+			   (uim-xemacs-make-event 
+			    (uim-convert-char-to-symbolvector 
+			     (replace-match "" nil nil key-str)))
+			 x)))))
+	   (append input-vector nil))))
 
 
 ;;
@@ -447,12 +548,11 @@
 	(uim-xemacs
 	 (or (equal keyvec (vector (uim-xemacs-make-event [escape])))
 	     (and (eq (global-key-binding keyvec) esc-map)
-		  (keymapp (uim-getbind keyvec)))))
+		  (keymapp (uim-key-binding keyvec)))))
 	))
 	 
 
 (defun uim-is-start-with-escape (keyvec)
-;  (uim-debug (format "uim-is-start-width-escape: %s" keyvec))
   (cond (uim-emacs
 	 (uim-is-single-escape (vector (aref keyvec 0))))
 	(uim-xemacs
@@ -461,7 +561,6 @@
 
 
 (defun uim-is-escape (keyvec)
-;  (uim-debug (format "uim-is-escape %s" keyvec))
   (cond (uim-emacs
 	 (if (or window-system
 		 (and (not window-system) uim-use-single-escape-on-terminal))
@@ -470,190 +569,136 @@
 	 )
 	(uim-xemacs
 	 (if (or window-system
-		 (and (not window-system) uim-use-single-escape-on-terminal))
+		 (and (not window-system) 
+		      uim-use-single-escape-on-terminal))
 	     (uim-is-single-escape keyvec)
-	   (or (equal keyvec (make-vector 2 (uim-xemacs-make-event [(escape)])))
-	       (equal keyvec (vector (uim-xemacs-make-event (uim-convert-char-to-symbolvector "M-ESC"))))
-	       )
-	   )
-	 )
-	)
+	   (or (equal keyvec 
+		      (make-vector 2 (uim-xemacs-make-event [(escape)])))
+	       (equal keyvec 
+		      (vector (uim-xemacs-make-event [(meta escape)]))))
+	   ))))
+
+(defun uim-separate-prefix-vector (key-vector)
+  (let (key-vector-prefix key-vector-main)
+    (setq key-vector-main (uim-last-onestroke-key key-vector))
+    (setq key-vector-prefix 
+	  (uim-get-vector-from-head key-vector
+				    (- (length key-vector)
+				       (length key-vector-main))))
+    ;;(setq key-vector key-vector-main)
+    ;;(setq uim-prefix-arg-vector key-vector-prefix)
+    (list key-vector-main key-vector-prefix))
   )
 
 
 ;;
 ;; get this-command-keys with symbol lists vector
 ;; 
-(defun uim-this-command-keys (with-arg)
-  (let (keyvec replace-continue fmap-continue)
+(defun uim-translate-key (input-vector)
+  (let (translated-vector map 
+	(input-vector-main input-vector)
+	(input-vector-prefix nil)
+	translated bind)
 
-    (if uim-xemacs
-	(setq keyvec (this-command-keys)))
+    (catch 'fmap-loop
+      (while input-vector-main
 
-    (if uim-emacs
-	(setq keyvec (this-command-keys-vector)))
+	(setq bind (uim-key-binding input-vector-main))
+	  
+	(if (and bind
+		 (not (integerp bind)))
+	    (progn
+	      ;;(setq translated-vector input-vector)
+	      (setq translated-vector nil)
+	      (throw 'fmap-loop t))
+	  
+	  (setq translated (lookup-key function-key-map input-vector-main))
 
-    (if uim-retry-keys
-	;; retry mode: ex. S-return -> return
-	(progn
-	  (uim-debug "use retry key")
-	  (setq keyvec uim-retry-keys)
-	  (setq uim-retry-keys nil)
-	  )
-      (if uim-prefix-ignore-next
-	  ;; avoid mysterious key event on Emacs-21 on terminal
-	  ;;  ex. C-u 1 0 escape f 
-	  (progn
-	    (uim-debug "ignore this key vector")
-	    (setq keyvec nil)
-	    (setq uim-prefix-ignore-next nil)
-	    )
-	(if with-arg
-	    ;; key with prefix arg
-	    (let* ((rkeylist (reverse (append keyvec nil))))
-	      (setq keyvec (uim-last-onestroke-key keyvec))
-	      (if (= (length keyvec) 2)
-		  ;; Only Emacs-21 returns two stroke keys at 1st time
-		  (setq uim-prefix-arg-vector 
-			(vconcat (reverse (cdr (cdr rkeylist)))))
-		(setq uim-prefix-arg-vector 
-		      (vconcat (reverse (cdr rkeylist)))))
-	      ;; work around
-	      (if (= (length keyvec) 2)
-		  (setq uim-prefix-ignore-next t))
-	      ))))
-
-    (uim-debug (format "keyvec %s" keyvec))
-
-    ;; workaround for Emacs22
-    ;;  detect and convert odd double key vector into single
-    ;;  vector (ex. [1 1] to [1])
-    (if (and  (>= (length  keyvec) 2)
-	      (not (uim-getbind keyvec)))
-	(progn
-	  (uim-debug "*** wrong key vector detected (Emacs22's bug?)")
-	  (setq keyvec (vector (aref keyvec 0)))))
-
-    ;; translate key vector with function-key-map
-
-    (let (fmap key replaced)
-      (let* ((merged-vector (vconcat uim-stacked-key-vector keyvec))
-	     (merged-list (append merged-vector nil))
-	     (stacked-list nil)
-	     done)
-	(uim-debug (format "merged-list: %s" merged-list))
-	(while (and (not done)
-		    merged-list)
-	  (setq fmap (lookup-key function-key-map (vconcat merged-list)))
-
-	  (if (and (or (not fmap) 
-		       (integerp fmap))
+	  (if (and (or (not translated)
+		       (integerp translated))
 		   (boundp 'local-function-key-map))
-	      ;; for Emacs multi-tty
-	      (setq fmap (lookup-key local-function-key-map
-				     (vconcat merged-list))))
+	      (setq translated (lookup-key local-function-key-map 
+					   input-vector-main)))
 
-	  (cond ((vectorp fmap)
-		 ;; vector: replace
-		 (uim-debug (format "vector: %s" (vconcat merged-list)))
-		 (setq uim-stacked-key-vector 
-		       (vconcat stacked-list fmap))
-		 (setq done t)
+	  (if (and (symbolp translated) (commandp translated))
+	      (let ((sf (symbol-function translated)))
+		(if (and (listp sf)
+			 (eq 'autoload (nth 0 sf))
+			 (or (eq 'keymap (nth 4 sf))
+			     (eq 'macro  (nth 4 sf))))
+		    (load (nth 1 sf)))))
+
+	  (cond ((not translated)
 		 )
-		((keymapp fmap)
-		 ;; keymap: wait next
-		 (uim-debug (format "keymap: %s" (vconcat merged-list)))
-		 (setq fmap-continue t)
-		 (setq uim-stacked-key-vector merged-vector)
-		 (setq done t)
-		 )
-		((functionp fmap)
-		 ;; function: Where should I call this function???
-		 ;;   Perhaps, this part doesn't work.
-		 (uim-debug (format "function: %s" (vconcat merged-list)))
-		 (setq fmap (fmap nil))
-		 (if (vectorp fmap)
-		     (setq uim-stacked-key-vector (vconcat stacked-list fmap))
+
+		((vectorp translated)
+		 ;; vector ... replace immediately
+		 (setq translated-vector 
+		       (vconcat input-vector-prefix translated))
+		 (throw 'fmap-loop t))
+
+		((keymapp translated)
+		 ;; keymap ... wait next input
+		 (setq map translated)
+		 (throw 'fmap-loop t))
+
+		((functionp translated)
+		 ;; function ... call immediately and use returned value
+		 (if (not uim-keystroke-displaying)
+		     (setq uim-keystroke-displaying (sit-for echo-keystrokes)))
+
+		 (if uim-keystroke-displaying
+		     (let (message-log-max)
+		       (message (concat (key-description 
+					 (vconcat uim-prefix-arg-vector
+						  input-vector))
+					"-")))
 		   )
-		 (setq done t)
-		 )
-		)
-	  (setq stacked-list (append stacked-list (list (car merged-list))))
-	  (setq merged-list (cdr merged-list))
-	  )
 
-	(if (not done)
-	    (setq uim-stacked-key-vector merged-vector))))
+		 (setq translated-vector
+		       (vconcat input-vector-prefix (funcall translated nil)))
+
+		 (throw 'fmap-loop t))
+		
+		((stringp translated)
+		 ;; string ... replace
+		 (setq translated-vector
+		       (char-to-string (aref translated 
+					     (- (length translated) 1))))
+		 (throw 'fmap-loop t))
+		))
+
+
+	(setq input-vector-prefix
+	      (vconcat input-vector-prefix (uim-vector-car input-vector-main)))
+
+	(setq input-vector-main
+	      (uim-vector-cdr input-vector-main))) ;; end of while
       
-    ;; Replate [escape escape] with [M-escape] on XEmacs
-    ;;   Some special keys cannot be used with escape key on terminal
-    (when (and uim-xemacs
-	       (>= (length uim-stacked-key-vector) 2)
-	       (equal (aref uim-stacked-key-vector 0) 
-		      (uim-xemacs-make-event [(escape)])))
-      ;; append meta
-      (setq uim-stacked-key-vector
-	    (vector
-	     (uim-xemacs-make-event
-	      (vector 
-	       (cons 'meta 
-		     (aref (uim-convert-char-to-symbolvector
-			    (key-description (vconcat
-					      (cdr (append 
-						    uim-stacked-key-vector
-						    nil)))))
-			   0)))))))
-
-
-
-    (uim-debug (format "stacked-key-vector: %s" uim-stacked-key-vector))
-
-    (cond ((and uim-preedit-keymap-enabled
-		(uim-is-escape uim-stacked-key-vector))
-	   (uim-debug "stacked-key is Escape")
-	   ;; Return escape key
-	   (if uim-emacs
-	       (setq keyvec [27]))
-	   (if uim-xemacs
-	       (setq keyvec (vector (uim-xemacs-make-event [escape]))))
-	   (setq uim-stacked-key-vector nil)
-	   )
-	  ((or (and uim-preedit-keymap-enabled
-		      (or 
-		       (and (or (eq (car-safe (aref uim-stacked-key-vector 0))
-				    'menu-bar)
-				(eq (car-safe (aref uim-stacked-key-vector 0))
-				    'tool-bar))
-			    (keymapp (uim-getbind uim-stacked-key-vector)))
-
-		    (and fmap-continue  ;; wait ESC- key vector
-			    (uim-is-start-with-escape uim-stacked-key-vector))
-		       ))
-	       
-	       (and (not uim-preedit-keymap-enabled)
-		    (or (and fmap-continue 
-			     (not (commandp (uim-getbind uim-stacked-key-vector))))
-			(keymapp (uim-getbind uim-stacked-key-vector))))
-	       uim-prefix-ignore-next ;; work around for Emacs-21 prefix arg
-	       )
-	   (uim-debug "wait next")
-	   (setq keyvec nil))
-	  (t
-	   ;; No need to wait any more. Return current keys.
-	   (setq keyvec uim-stacked-key-vector)
-	   (setq uim-stacked-key-vector nil))
-	  )
-	
-    ;; convert keyvector with key-translation-map
-    (if keyvec
-	(let ((transvec (lookup-key key-translation-map keyvec)))
-	  (if (vectorp transvec)
-	      (setq keyvec transvec))))
-
-    keyvec
-    )
+      (setq translated-vector nil)) ;; end of catch
+    
+    (list translated-vector map))
   )
 
 
+
+;; Replate [escape escape] with [M-escape] on XEmacs
+;;   Some special keys cannot be used with escape key on terminal
+(defun uim-translate-escape-meta (input-vector)
+  (if (and uim-xemacs
+	   (>= (length input-vector) 2)
+	   (equal (aref input-vector 0)
+		  (uim-xemacs-make-event [(escape)])))
+      ;; append meta
+      (vconcat 
+       (vector 
+	(uim-xemacs-make-event 
+	 (vector (cons 'meta (aref (uim-convert-char-to-symbolvector
+				   (key-description (uim-vector-cdr 
+						     input-vector))) 0)
+		      ))))
+       (uim-cut-vector-from-head input-vector 2))
+    input-vector))
+
 (provide 'uim-key)
 
