@@ -1,5 +1,5 @@
 ;; 
-;;  Copyright (c) 2005-2006 uim Project http://uim.freedesktop.org/
+;;  Copyright (c) 2005-2008 uim Project http://code.google.com/p/uim/
 ;;
 ;;  All rights reserved.
 ;;
@@ -130,7 +130,6 @@
 ;; Replace tabs to the same width white spaces.
 ;;
 (defun uim-replace-tab (start end)
-;  (uim-debug (format "uim-replace-tab %s %s" start end))
   (let (tpos (cnt 0) (ofs 0) tabwidth)
     (save-excursion
       (goto-char start)
@@ -160,7 +159,6 @@
 ;; Replace tabs and spaces.
 ;;
 (defun uim-tab-pad-space (start end)
-  (uim-debug (format "uim-tab-pad-space %s %s" start end))
   (let (org lstart lend (endorg end) tabspace)
     (setq org (point))
     (save-excursion
@@ -299,10 +297,44 @@
 
 
 (defun uim-goto-char (pt)
-  (uim-debug (format "uim-goto-char: %s" pt))
   (set-window-point (get-buffer-window (current-buffer)) pt))
 
-(defun uim-point ()
-  (window-point (get-buffer-window (current-buffer))))
+
+(defun uim-get-vector-from-head (vec n)
+  (if (and (<= n (length vec))
+	   (> n 0))
+      (let ((vlist (append vec nil))
+	    (ret '()))
+	(while (> n 0)
+	  (setq ret (append ret (list (car vlist))))
+	  (setq vlist (cdr vlist))
+	  (setq n (- n 1)))
+	(vconcat ret))))
+
+(defun uim-get-vector-from-tail (vec n)
+  (if (and (<= n (length vec))
+	   (> n 0))
+      (vconcat (nthcdr (- (length vec) n) (append vec nil)))))
+
+(defun uim-cut-vector-from-head (vec n)
+  (if (<= n (length vec))
+      (uim-get-vector-from-tail vec (- (length vec) n))))
+
+
+(defun uim-vector-cdr (vec)
+  (if (> (length vec) 1)
+      (uim-get-vector-from-tail vec (- (length vec) 1))))
+
+(defun uim-vector-car (vec)
+  (uim-get-vector-from-head vec 1))
+
+(defun uim-delete-atom (list)
+  (if (and list 
+	   (not (atom list)))
+      (if (and (car list) 
+	       (atom (car list)))
+	  (uim-delete-atom (cdr list))
+	(cons (car list) (uim-delete-atom (cdr list))))
+    list))
 
 (provide 'uim-util)
