@@ -1,5 +1,5 @@
 ;; 
-;;  Copyright (c) 2005-2007 uim Project http://uim.freedesktop.org/
+;;  Copyright (c) 2005-2008 uim Project http://code.google.com/p/uim/
 ;;
 ;;  All rights reserved.
 ;;
@@ -114,8 +114,6 @@
 ;;
 (defun uim-update-default-engine (engine)
 
-;  (uim-debug (format "update-default-engine: %s" engine))
-
   (when (not (equal uim-default-im-engine engine))
     (setq uim-default-im-engine engine)
     (run-hooks 'uim-update-default-engine-hook)
@@ -129,8 +127,6 @@
 
   (setq engine (car engine))
 
-  (uim-debug (format "update-current-engine: %s" engine))
-
   (when (not (equal uim-current-im-engine engine))
     (setq uim-current-im-engine engine) 
     (run-hooks 'uim-update-current-engine-hook))
@@ -227,10 +223,6 @@
   ;; detect switching of current buffer
   (when (not (eq uim-recent-buffer (current-buffer)))
 
-    (uim-debug (format "current-buffer changed (from %s to %s)"
-		       uim-focused-buffer
-		       (current-buffer)))
-
     ;; update recent bufferto detect next change
     (setq uim-recent-buffer (current-buffer))
     
@@ -240,18 +232,13 @@
 	(save-current-buffer
 	  (set-buffer uim-focused-buffer)
 	(when (and uim-initialized uim-mode)
-	  (uim-debug (format "unfocused %s" (buffer-name (current-buffer))))
 	    (uim-unfocused)
 	    )))
 	    
     ;; set Uim focus to current buffer if it has Uim context
     (when (and uim-initialized uim-mode)
-      (uim-debug (format "focused %s" (buffer-name (current-buffer))))
 	  (uim-focused)
 	  )
-
-    (uim-debug (format "current-buffer %s / uim-focused-buffer %s" 
-		       (current-buffer) uim-focused-buffer ))
     )
   )
 
@@ -276,7 +263,6 @@
 ;; Update process encoding 
 ;; 
 (defun uim-change-process-encoding (outcode)
-  (uim-debug (format "set-process-encoding to %s" outcode))
   (set-process-coding-system uim-el-agent-process outcode 'iso-8859-1)
   )
 
@@ -328,8 +314,6 @@
 ;;
 (defun uim-init-buffer ()
 
-  (uim-debug (format "initialize buffer: %s" (current-buffer)))
-
   ;; if buffer has no context
   (setq uim-context-id (uim-context-create))
 
@@ -374,8 +358,6 @@
 ;;
 (defun uim-mode-on ()
 
-  (uim-debug "uim-mode-on")
-
   (if uim-mode
       (message "uim.el: uim-mode is already activated. (buffer %s)" 
 	       (current-buffer)))
@@ -383,6 +365,11 @@
   ;; Initialize IM encoding
   (uim-im-init)
 
+  ;; override some functions
+  (uim-this-command-keys-override)
+  (if (= emacs-major-version 21)
+      (uim-read-char-exclusive-override))
+
   (if (not buffer-read-only)
       (progn 
 
@@ -423,11 +410,13 @@
   ;; update mode-line
   (force-mode-line-update)
 
-
   ;; Disable uim-mode
   (setq uim-mode nil)
 
-  (uim-debug (format "uim-mode: %s" uim-mode))
+  ;; restore original functions
+  (uim-this-command-keys-restore)
+  (if (= emacs-major-version 21)
+      (uim-read-char-exclusive-restore))
 
   )
 
@@ -438,9 +427,6 @@
 ;;
 (defun uim-kill-buffer ()
 
-  (uim-debug (format "kill-bufer %s"
-		     (buffer-name (current-buffer))))
-
   ;; unfocus
   (if (eq uim-focused-buffer (current-buffer))
       (uim-unfocused))
@@ -455,8 +441,6 @@
 ;;
 (defun uim-change-major-mode ()
 
-  (uim-debug "uim-change-major-mode")
-
   (when uim-initialized
        (uim-unfocused)
        (uim-delete-context))
@@ -528,7 +512,6 @@
 ;;
 (defun uim-restore-undo ()
   (when uim-buffer-undo-list-saved
-    (uim-debug "restore undo list")
     (buffer-enable-undo)
     (setq buffer-undo-list uim-buffer-undo-list)
     (setq uim-buffer-undo-list-saved nil))
@@ -540,7 +523,6 @@
 ;;
 (defun uim-save-undo ()
   (when (not uim-buffer-undo-list-saved)
-    (uim-debug "save undo list")
     (setq uim-buffer-undo-list buffer-undo-list)
     (buffer-disable-undo)
     (setq uim-buffer-undo-list-saved 't))
@@ -670,8 +652,6 @@
 ;; 
 (defun uim-send-cmd (cmd serial)
 
-  (uim-debug (concat (format "%s " serial) cmd " (" (current-time-string) ")"))
-
   ;; check process status and start if it's not running
   (if (not uim-el-agent-process)
       (setq uim-el-agent-process (uim-el-agent-start)))
@@ -681,6 +661,9 @@
 	(save-current-buffer
 	  (set-buffer uim-el-agent-buffer)
 	  (erase-buffer))
+
+        (uim-debug (format "%s"
+                           (concat (format "%s " serial) cmd)))
 	
 	(process-send-string uim-el-agent-process
 			     (concat (format "%s " serial) cmd "\n"))
@@ -733,8 +716,6 @@
 
     (set-process-filter uim-el-helper-agent-process nil)
 
-    (uim-debug "wait recv")
-
     (let ((start (nth 1 (current-time)))
 	  (accept uim-el-agent-accept-timeout))
       (catch 'read-loop
@@ -792,17 +773,17 @@
 	(uim-wait-recv serial))))
 
 
-;;
-;; Save face property
-;;
-(defun uim-get-face-list (start end)
+;; for XEmacs
+(defun uim-overwrite-font-face (start end)
   (let ((facelist '()) tail face)
     (catch 'face-loop
       (while t
 	(setq tail
 	      (next-single-property-change start 'face (current-buffer) end))
 	(if (setq face (get-text-property start 'face))
-	    (setq uim-facelist (cons (list face start tail) uim-facelist)))
+	    (if (atom face)
+		(put-text-property start tail 'face 
+				   (cons face nil))))
 	(setq start tail)
 	(if (= start end)
 	    (throw 'face-loop t))))
@@ -810,114 +791,51 @@
   )
 
 
-
-;;
-;; Save face property of the window
-;;   Save only displayed region
-(defun uim-get-facelist-window (win)
-  (select-window win)
-  (let (end)
-    (save-excursion
-      (goto-char (window-end nil t))
-      (vertical-motion (window-height))
-      (setq end (point)))
-    (save-excursion
-      (uim-get-face-list (window-start) end)
-      )
-    ))
-
-
-;;
-;; Save face property of windows which are displaying the current buffer.
-;;
-(defun uim-get-facelist-buffer ()
-  (save-selected-window
-    (mapcar 'uim-get-facelist-window
-	    (get-buffer-window-list (current-buffer) t 'visible))
-    uim-facelist))
-
-
-;;
-;; Disable font-lock mode
-;;    This function should be called only when the font-lock-mode is 
-;;   already in use
-;;
-(defun uim-disable-font-lock-mode ()
-  ;; save and disable font-lock-mode
-  (setq uim-font-lock-mode font-lock-mode)
-  (setq uim-font-lock-verbose font-lock-verbose)
-  (when font-lock-mode
-    (font-lock-mode 0)
-    (setq font-lock-verbose nil)))
-
-
-;;
-;; Eable font-lock mode
-;;
-(defun uim-enable-font-lock-mode ()
-  (when uim-font-lock-mode
-    (font-lock-mode uim-font-lock-mode)
-    (setq font-lock-verbose uim-font-lock-verbose)))
-
-
-;;
-;; Restore face property
-;;
-(defun uim-put-face (facelist)
-  (put-text-property (nth 1 facelist) (nth 2 facelist)
-		     'face (car facelist)))
-
-
-;;
-;; Update font 
-;;
-(defun uim-update-font ()
-  (mapcar 
-   '(lambda (x)
-      (let ((font (or (cdr (assq 'font (frame-parameters)))
-		      (face-font 'default))))
-	(set-face-font x font)))
-		     
-   '(uim-preedit-face 
-     uim-preedit-underline-face 
-     uim-preedit-highlight-face
-     uim-preedit-highlight-underline-face
-     uim-separator-face
-     uim-candidate-odd-face
-     uim-candidate-even-face
-     uim-candidate-selected-face
-     uim-candidate-nth-face)))
-
-;;
-;; Lock appearance of buffer
-;;
 (defun uim-freeze-buffer ()
-  (let (facelists)
+  (when (not uim-buffer-frozen)
+    (setq uim-after-change-functions after-change-functions)
+    (setq after-change-functions nil)
 
-    (setq uim-facelist nil)
-    (uim-get-facelist-buffer)
-    ;(setq facelists (uim-get-face-list 1 (point-max)))
-    (if (memq 'font-lock features)	     
-	(progn
-	  (uim-disable-font-lock-mode)
-	  (mapcar 'uim-put-face uim-facelist)))
-    )
-  (setq uim-buffer-frozen t))
+    (when (boundp 'timer-idle-list)
+      (setq uim-timer-idle-list timer-idle-list)
+      (setq timer-idle-list nil))
+
+    (when (boundp 'fontification-functions)
+      (setq uim-fontification-functions fontification-functions)
+      (setq fontification-functions nil))
+
+    (setq uim-buffer-frozen t))
+  )
 
 
-
-;;
-;; Unlock appearance of buffer
-;;
 (defun uim-unfreeze-buffer ()
-  (let (facelists)
-    (if (memq 'font-lock features)
-	(uim-enable-font-lock-mode))
-    )
-  (setq uim-buffer-frozen nil))
+  (when uim-buffer-frozen ;; avoid multiple unfreeze 
+    (setq after-change-functions uim-after-change-functions)
+    (setq uim-after-change-functions nil)
+
+    (when (boundp 'timer-idle-list)
+      (setq timer-idle-list uim-timer-idle-list)
+      (setq uim-timer-idle-list nil))
+
+    (when (boundp 'fontification-functions)
+      (setq fontification-functions uim-fontification-functions)
+      (setq uim-fontification-functions nil))
+
+    (setq uim-buffer-frozen nil)))
 
  
 
+;;
+;; Reset some global parameters for inputted key processing
+;;
+(defun uim-reset-input-parameter ()
+  (setq uim-translated-key-vector nil)
+  (setq uim-untranslated-key-vector nil)
+  (setq uim-prefix-arg nil)
+  (setq uim-prefix-arg-vector nil)
+  (setq uim-merge-next nil))
+
+
 ;;
 ;; Process inputted key
 ;;
@@ -926,173 +844,275 @@
 		     last-input-event
 		     ))
 
-  (if uim-emacs
-      (setq uim-deactivate-mark t))
+  (let (new-key-vector send-vector send-vector-raw issue-vector
+        send issue mouse wait discard
+	(critical t))
 
-  (if (not uim-stacked-key-vector)
-      (setq uim-show-keystrokes nil))
-
-  ;; send an input key code to uim-el-agent
-
-  (if current-prefix-arg
-      (setq uim-prefix-arg current-prefix-arg))
-
-  
-  (if uim-prefix-arg
-      (uim-debug (format "uim-prefix-arg %s" uim-prefix-arg)))
-
-
-  (let ((keyvec (uim-this-command-keys current-prefix-arg))
-	sendkey newvec bypass count mouse)
-
-    (if uim-prefix-arg
-	(setq current-prefix-arg uim-prefix-arg))
-
-    (if keyvec
+    (unwind-protect
 	(progn
-	  (if uim-show-keystrokes
-	      (let (message-log-max)
-		(message (key-description keyvec))))
 
-	  (setq uim-last-key-vector keyvec)
+	  (if uim-xemacs
+	      (setq zmacs-region-stays nil))
 
-	  (cond (uim-xemacs 
-		 ;; XEmacs ;;
-		 (setq sendkey 
-		       (uim-convert-keystr-to-uimagent-vector (key-description keyvec)))
+	  (if (not (or uim-translated-key-vector uim-untranslated-key-vector))
+	      (setq uim-keystroke-displaying nil))
 
-		 (cond ((string-match "button\\(1\\|2\\|3\\|4\\|5\\)" 
-				      (key-description keyvec))
-			;; through mouse event
-			(setq mouse t)
-			(setq bypass t))
-		       
-		       (current-prefix-arg
-			(setq bypass t)
-			;;(setq count (prefix-numeric-value arg))
-			(setq count (prefix-numeric-value current-prefix-arg))
-			(uim-debug (format "with prefix arg: count is %s" count))
-			)
-		       
-		       ((and (not (or uim-preedit-displayed 
-				      uim-candidate-displayed))
-			     (uim-buffer-not-editable))
-			;; through uim-el-agent when buffer is uneditable
-			(setq bypass t)
-			))
-		 
-		 (if bypass
-		     (setq sendkey 
-			   (uim-convert-char-to-symbolvector 
-			    (key-description keyvec))))
-		 )
+	  (setq new-key-vector (uim-this-command-keys-vector))
+
+	  (if (or current-prefix-arg
+		  uim-merge-next)
+	      (let (vector-list)
+		(setq vector-list (uim-separate-prefix-vector new-key-vector))
+		(setq new-key-vector (car vector-list))
+		(setq uim-prefix-arg-vector (nth 1 vector-list))
+
+		;; Workaround for FSF Emacs-20/21
+		;;  Key event beginning with C-u and terminating with 
+		;;  ESC-something invokes uim-process-input with twice 
+		;;  at a time.
+		;;  In such a case, uim.el merges these two inputs.
+		(if (not uim-merge-next)
+		    (progn
+		      ;; normal
+		      (setq uim-prefix-arg current-prefix-arg)
+		      (if (and uim-emacs
+			       (not window-system)
+			       (<= emacs-major-version 21)
+			       (>= (length new-key-vector) 2))
+			  ;; workaround
+			  (setq uim-merge-next t)))
+
+		  ;; uim-merge-next is valid
+		  (setq uim-merge-next nil)
+		  (setq uim-translated-key-vector nil)
+		  (setq uim-untranslated-key-vector nil))
 		
-		(uim-emacs 
-		 ;; Emacs ;;
-		 (setq sendkey keyvec)
-		 
-		 (cond (current-prefix-arg
-			(setq bypass t)
-			(setq count (prefix-numeric-value arg))
-			(setq count (prefix-numeric-value current-prefix-arg))
-			(uim-debug (format "with prefix arg: count is %s"
-					   count))
-			)
-		       ((and (eventp event)
-			     (memq (event-basic-type event) 
-				   '(mouse-1 mouse-2 mouse-3 mouse-4 mouse-5)))
-			;; detect mouse event
-			;;(uim-debug "mouse event")
-			(setq bypass t)
-			(setq mouse t)
-			)
-		       ((and (not (or uim-preedit-displayed
-				      uim-candidate-displayed))
-			     (uim-buffer-not-editable))
-			(setq bypass t))
-		       (t 
-			;; overwrite special keyvector
-			(cond ((equal sendkey [127])   (setq sendkey [backspace]))
-			      ((equal sendkey [27 27]) (setq sendkey [27]))
-			      ((equal sendkey [28])    (setq sendkey [C-\\]))
-			      ((equal sendkey [29])    (setq sendkey [C-\]]))
-			      ((equal sendkey [30])    (setq sendkey [C-~]))
-			      ((equal sendkey [31])    (setq sendkey [C-_]))
-			      ))
-		       )
-		 ))
+		))
+
+	  (if uim-xemacs
+	      (setq uim-original-input-event (copy-event last-input-event)))
+
+	  (setq uim-untranslated-key-vector
+		(vconcat uim-untranslated-key-vector new-key-vector))
 	  
-	  ;; pass only Uim supported keys
-	  (when (or (>= (length sendkey) 3)
-		    (and (= (length sendkey) 2)
-			 (not (or (eq (aref sendkey 0) 27)
-				  (eq (aref sendkey 0) 'escape)))))
-	    (uim-debug "bypass")
-	    (setq bypass t))
+	  (let (translated-list translated map)
+	    
+	    (setq translated-list 
+		  (uim-translate-key uim-untranslated-key-vector))
+	    (setq translated (nth 0 translated-list))
+	    (setq map (nth 1 translated-list))
+
+	    (cond ((stringp translated)
+		   (setq issue t)
+		   (setq uim-translated-key-vector translated)
+		   (setq uim-untranslated-key-vector nil))
+
+		  ((vectorp translated)
+		   (setq send t) 
+		   (setq uim-translated-key-vector 
+			 (vconcat uim-translated-key-vector translated))
+		   (setq uim-untranslated-key-vector nil))
+
+		  (map
+		   (setq wait t))
+
+		  (t
+		   (setq send t))
+		  ))
+
+	  (when send
+
+	    (setq send-vector-raw (vconcat uim-translated-key-vector
+					   uim-untranslated-key-vector))
+
+	    (setq send-vector 
+		  (if uim-emacs 
+		      send-vector-raw
+		    (if uim-xemacs
+			(uim-convert-keystr-to-uimagent-vector (key-description send-vector-raw))
+		      nil)))
+
+	    (cond (uim-merge-next
+		   (setq send nil))
+
+		  ((or (and uim-emacs 
+			    (eventp event)
+			    (memq (event-basic-type event) 
+				  '(mouse-1 mouse-2 mouse-3 mouse-4 mouse-5)))
+		       (and uim-xemacs 
+			    (string-match "button\\(1\\|2\\|3\\|4\\|5\\)" 
+					  (key-description send-vector-raw))))
+		   (setq send nil)
+		   (setq mouse t))
+
+		  ((or (and uim-preedit-keymap-enabled
+			    (not window-system)
+			    (not uim-use-single-escape-on-terminal)
+			    (uim-is-single-escape send-vector-raw)))
+		   (setq send nil)
+		   (setq wait t))
+
+		  ((or current-prefix-arg
+		       uim-prefix-arg)
+		   (setq send nil)
+		   (setq issue t))
+		  
+		  ((or (eq (car-safe (aref send-vector 0)) 'menu-bar)
+		       (eq (car-safe (aref send-vector 0)) 'tool-bar))
+		   (setq send nil)
+		   (if (not uim-preedit-keymap-enabled)
+		       (setq issue t)
+		     (if (and uim-preedit-keymap-enabled
+			      (keymapp (uim-key-binding send-vector)))
+			 (setq wait t)
+		       (setq discard t))))
+
+		  ((and (or (>= (length send-vector) 3)
+			    (and (= (length send-vector) 2)
+				 (not (or (eq (aref send-vector 0) 27)
+					  (eq (aref send-vector 0) 'escape))))))
+		   (setq send nil)
+		   (if (not uim-preedit-keymap-enabled)
+		       (setq issue t)))
+
+		  ))
 
 
-	  (setq uim-retry-keys nil)
+	  (when send
 
-	  (unwind-protect
-	      (if (and uim-preedit-keymap-enabled
-		       (or (eq (car-safe (aref sendkey 0)) 'menu-bar)
-			   (eq (car-safe (aref sendkey 0)) 'tool-bar)))
-		  (uim-do-send-recv-cmd (format "%d NOP" uim-context-id))
-	  (if (not bypass)
-	      (uim-do-send-recv-cmd (format "%d %s" uim-context-id sendkey))
-	    (if mouse
-		(uim-process-mouse-event event)
-	      (uim-process-keyvec uim-last-key-vector count)
+	    (setq uim-last-key-vector send-vector-raw)
 
-	      ;; display "ESC ESC ESC" or something
+	    (when uim-emacs
+	      (cond ((equal send-vector [127])   (setq send-vector [backspace]))
+		    ((equal send-vector [27 27]) (setq send-vector [27]))
+		    ((equal send-vector [28])    (setq send-vector [C-\\]))
+		    ((equal send-vector [29])    (setq send-vector [C-\]]))
+		    ((equal send-vector [30])    (setq send-vector [C-~]))
+		    ((equal send-vector [31])    (setq send-vector [C-_]))
+		    ))
+	    
+	    (when uim-xemacs 
 
-	      (if uim-show-keystrokes
-		  (let (message-log-max keymsg)
-		    (setq keymsg (if uim-prefix-arg-vector
-				     (concat (key-description uim-prefix-arg-vector)
-					     " ")
-				   nil))
-		    (mapcar '(lambda (x)
-			       (setq keymsg (concat keymsg
-						(key-description (vector x))
-						" ")))
-			    (append uim-last-key-vector nil))
-		    (message keymsg))
-		)
-		    )))
-	  
-	    (progn
-	  (uim-debug "reset prefix variables")
-	  (setq uim-prefix-arg nil)
-	  (setq uim-prefix-arg-vector nil)
-	  (setq uim-prefix-ignore-next nil)
-	      )
+	      (if (equal (make-vector 2 (uim-xemacs-make-event [(escape)]))
+			 send-vector-raw)
+		  (setq send-vector-raw
+			(vector (uim-xemacs-make-event [(escape)]))))
+
+	      (setq send-vector-raw (uim-translate-escape-meta send-vector-raw))
+
+	      (setq send-vector 
+		    (uim-convert-keystr-to-uimagent-vector 
+		     (key-description send-vector-raw))))
+
+	    (setq uim-wait-next-key nil)
+	    (uim-do-send-recv-cmd (format "%d %s" 
+					  uim-context-id send-vector))
+	    (setq wait uim-wait-next-key)
+		
+	    (when (not uim-wait-next-key)
+	      (uim-reset-input-parameter)))
+	    
+	  (when mouse
+	    (uim-process-mouse-event event)
+	    (uim-reset-input-parameter))
+
+	  (when issue
+	    (let (issue-vector-raw)
+	      (setq issue-vector-raw (vconcat uim-translated-key-vector
+					      uim-untranslated-key-vector))
+	      
+	      (setq issue-vector 
+		    (if uim-emacs 
+			issue-vector-raw
+		      (if uim-xemacs
+			  (uim-convert-char-to-symbolvector 
+			   (key-description issue-vector-raw)))))
+	      
+	      (setq uim-last-key-vector issue-vector-raw)
+      
+	      (setq wait
+		    (uim-process-key-vector issue-vector-raw uim-prefix-arg))  
+
+	      (when (not wait)
+		(uim-reset-input-parameter))))
+
+	  (when wait
+
+	    (if (and (memq last-command (list 'universal-argument
+					      'digit-argument
+					      'negative-argument))
+		     (current-message))
+		(setq uim-keystroke-displaying t))
+
+	    (if (not uim-keystroke-displaying)
+		(cond (uim-emacs
+		       (let (key)
+			 (when (setq key (with-timeout (echo-keystrokes nil)
+					   (read-key-sequence-vector nil)))
+			   (setq unread-command-events
+				 (nconc (listify-key-sequence key) 
+					unread-command-events)))
+			 
+			 (setq uim-keystroke-displaying (not key))))
+		      (uim-xemacs
+		       (setq uim-keystroke-displaying (sit-for echo-keystrokes)))))
+
+	    ;; display "ESC-" or something
+	    (if uim-keystroke-displaying
+		(let (message-log-max)
+		  (message (concat (key-description 
+				    (vconcat uim-prefix-arg-vector
+					     (if uim-xemacs
+						 (uim-translate-escape-meta uim-translated-key-vector)
+					       uim-translated-key-vector)
+					     (if uim-xemacs
+						 (uim-translate-escape-meta uim-untranslated-key-vector)
+					       uim-untranslated-key-vector)))
+				   (if uim-xemacs " ")
+				   "-"))))
+
+	    (if uim-emacs
+		(setq uim-deactivate-mark nil))
+
+	    (if uim-xemacs
+		(setq zmacs-region-stays t))
 	    )
 
-	  )
+	  (when discard
+	    (uim-reset-input-parameter))
 
-      ;; if keyvec is nil
-      (setq uim-deactivate-mark nil)
 
-      (if (not uim-show-keystrokes)
-	  (if (sit-for echo-keystrokes)
-	      (setq uim-show-keystrokes t)))
+	  ;; display "ESC ESC ESC" or something
+	  (when (and (or send issue)
+		     (not wait)
+		     uim-keystroke-displaying)
 
-      ;; display "ESC-" or something
-      (if uim-show-keystrokes
-	  (let (message-log-max)
-	    (message (concat (key-description (vconcat uim-prefix-arg-vector
-						       uim-stacked-key-vector))
-			     "-"))))
-      ))
+	    (let (message-log-max msg)
+	      (setq msg (if uim-prefix-arg-vector
+			    (concat " " (key-description uim-prefix-arg-vector) " ")
+			  ""))
 
-  (if uim-emacs
-      (setq deactivate-mark uim-deactivate-mark))
+	      (mapcar '(lambda (x)
+			 (setq msg (concat msg
+					   (key-description (vector x)) " ")))
+		 
+		      (append (if uim-xemacs
+				  (uim-translate-escape-meta send-vector-raw)
+				uim-last-key-vector) nil))
+	      (message msg)))
+
+	  (if uim-emacs
+	      (setq deactivate-mark uim-deactivate-mark))
+
+	  (setq critical nil))
+
+      (when critical
+	(uim-reset-input-parameter)))
+    )
   )
 
 
 
-
 (defun uim-enter-preedit-mode ()
   ;; change keymap and freeze faces at first time
   (uim-enable-preedit-keymap)
@@ -1104,8 +1124,8 @@
 
 (defun uim-leave-preedit-mode ()
   (uim-disable-preedit-keymap)
-  (uim-unfreeze-buffer)
   (setq buffer-read-only uim-buffer-read-only)
+  (uim-unfreeze-buffer)
   )
 
 
@@ -1113,15 +1133,14 @@
 ;; process return expression from uim-el-agent 
 (defun uim-process-agent-output (str)
   (catch 'process-agent-output
-    (let (;;(inhibit-quit t)
+    (let ((inhibit-quit t)
 	  preedit-existed
 	  candidate-existed
 	  key commit preedit candidate default im label imlist helpermsg
 	  )
-      
+
       (uim-debug (format "%s" str))
       
-
       (let ((modified (buffer-modified-p)))
 
 	;; remove candidate anyway
@@ -1137,8 +1156,7 @@
 	  (setq preedit-existed t)
 	  (let ((inhibit-read-only t))
 	    (uim-remove-preedit))
-	  (setq uim-preedit-displayed nil)
-	  (uim-goto-char uim-original-cursor))
+	  (setq uim-preedit-displayed nil))
 
 	;; restore cursor point
 	(when (and uim-preedit-keymap-enabled uim-original-cursor)
@@ -1154,9 +1172,7 @@
 	
 	    (cond ((string= rcode "n") ;; uim returns key code
 		   (if uim-last-key-vector
-		       (progn
-			 (setq key uim-last-key-vector)
-			 (uim-debug (format "last-key: %s" key)))
+		       (setq key uim-last-key-vector)
 		     (setq key (car rval)))
 		   )
 		  ((string= rcode "s") ;; commit string
@@ -1213,34 +1229,31 @@
 
 	    (mapcar
 	     '(lambda (x)
-		(let ((start (point)))
-		  
-		  ;; enable buffer-undo temporarily
-		  (when uim-buffer-undo-list-saved
-		    (setq buffer-undo-list nil)
-		    (buffer-enable-undo))
+		;; enable buffer-undo temporarily
+		(when uim-buffer-undo-list-saved
+		  (setq buffer-undo-list nil)
+		  (buffer-enable-undo))
 
-		  (insert x)
-		  (uim-debug (format "insert %s" x))
+		(let ((buffer-undo-list-tmp buffer-undo-list))
+		  (unwind-protect
+		      (progn
+			(setq buffer-undo-list nil)
+			(insert x))
+		    (when buffer-undo-list
+		      (setq buffer-undo-list
+ 			    (append (cons nil 
+					  (uim-delete-atom buffer-undo-list))
+				    buffer-undo-list-tmp))
 
-		  ;; disable buffer-undo temporarily
-		  (when uim-buffer-undo-list-saved
-		    (let (delatom)
-		      (defun delatom (x)
-			(if (and x (not (atom x)))
-			    (if (and (car x) (atom (car x)))
-				(delatom (cdr x))
-			      (cons (car x) (delatom (cdr x))))
-			  x))
-		      (setq buffer-undo-list (delatom buffer-undo-list))
-		      )
+		      )))
 
-		    (setq uim-buffer-undo-list
-			  (append buffer-undo-list uim-buffer-undo-list))
-		    (setq buffer-undo-list nil)
-		    (buffer-disable-undo))
-		  )
-		)
+		;; disable buffer-undo temporarily
+		(when uim-buffer-undo-list-saved
+		    
+		  (setq uim-buffer-undo-list
+			(append buffer-undo-list uim-buffer-undo-list))
+		  (setq buffer-undo-list nil)
+		  (buffer-disable-undo)))
 	     commit)
 
 	    (if auto-fill-function
@@ -1256,19 +1269,30 @@
 	  (let (keyproc-done
 		(buffer-undo-list-saved uim-buffer-undo-list-saved))
 	    (when buffer-undo-list-saved
-	      (uim-debug "call restore undo 2")
 	      (uim-restore-undo))
 	    ;; process raw key
 	    ;;  C-o is also processed here ... orz
 	    (let ((inhibit-read-only t))
 	      (unwind-protect
 		  (progn
-		    (uim-process-keyvec key)
+
+		    ;; remove shift if possible
+		    (if (and (uim-check-shift key)
+			     (not (uim-key-binding key)))
+			;; with shift key but no key bind
+			(let* ((keytmp (uim-remove-shift key))
+			       (translated (nth 0 (uim-translate-key keytmp))))
+			  ;; lookup function-key-map
+			  (if translated
+			      (setq key translated)
+			    (if (uim-key-binding keytmp)
+				(setq key keytmp)))))
+
+		    (if (uim-process-key-vector key uim-prefix-arg)
+			(setq uim-wait-next-key t))
 		    (setq keyproc-done t))
 		(when (not keyproc-done)
-		  (uim-leave-preedit-mode)
-		  (uim-debug "leave preedit mode forcibly")
-		  )
+		  (uim-leave-preedit-mode))
 		)
 	      ;; following expressions will not be evaluated
 	      ;; when an error occurs in this function...
@@ -1277,10 +1301,9 @@
 		(throw 'process-agent-output t))
 	    ;; save undo hisotry again
 	    (when buffer-undo-list-saved
-	      (uim-debug "call save undo 2")
 	      (uim-save-undo))))
 
-      (setq uim-original-cursor (uim-point))
+      (setq uim-original-cursor (point))
 
 
       (if (or preedit candidate)
@@ -1290,7 +1313,6 @@
 	    ;; save undo list if not saved
 	    (when (not uim-buffer-undo-list-saved)
 	      (uim-flush-concat-undo)
-	      (uim-debug "call save undo 3")
 	      (uim-save-undo))
 
 	    ;; change keymap and freeze faces at first time
@@ -1311,7 +1333,7 @@
 
 	    (uim-goto-char uim-preedit-cursor)
 
-	    (setq uim-candidate-cursor (uim-point))
+	    (setq uim-candidate-cursor (point))
 
 	    ;; show candidate
 	    (when (and candidate
@@ -1332,7 +1354,6 @@
 
 	;; restore undo-list if saved
 	(when uim-buffer-undo-list-saved
-	  (uim-debug "call restore undo 3")
 	  (uim-restore-undo))
 
 	;; no raw-key and no preedit/candidate
@@ -1346,19 +1367,17 @@
       (when (and candidate-existed
 		 (not uim-candidate-displayed) 
 		 uim-window-force-scrolled) 
-	(uim-debug "recenter: window-force-scrolled is true")
 	(setq uim-window-force-scrolled nil)
-	(recenter))
-
+	(save-excursion
+	  (goto-char uim-window-force-scrolled-original)
+	  (recenter 0))
+	(setq uim-window-force-scrolled-original nil))
 
       (if (not uim-send-recv-again)
 	  (when label 
 	    (setq uim-send-recv-again t)
 	    (uim-update-im-label))
 	(setq uim-send-recv-again nil))
-
-      (uim-debug (format "uim-original-cursor %s" uim-original-cursor))
-      (uim-debug "process-agent-output done")
       ))
   )
 
@@ -1399,9 +1418,9 @@
 (defun uim-minibuffer-exit ()
   (if (and uim-initialized
 	   uim-mode)
-      ;(uim-mode-off)
+	;;(uim-mode-off)
       (uim-force-off)
-      ))
+    ))
 
 
 
@@ -1453,7 +1472,8 @@ uim mode facilitates internationalized input through the uim library."
     (if uim-mode
 	(uim-mode-off)
       (uim-mode-on))))
-; compat
+
+;; compat
 (defun uim-mode-switch ()
   (interactive)
   (if uim-mode
@@ -1519,6 +1539,7 @@ uim mode facilitates internationalized input through the uim library."
 ;; Initialize uim.el 
 (uim-init)
 
+(run-hooks 'uim-load-hook)
 
 (provide 'uim)
 
