@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -50,13 +50,12 @@
 #include <unistd.h>
 #include <locale.h>
 
-#include "uim-stdint.h"
 #include "uim/uim.h"
 #include "uim/uim-util.h"
 #include "uim/uim-helper.h"
 #include "uim/uim-im-switcher.h"
 #include "uim/gettext.h"
-#include "uim/uim-compat-scm.h"
+#include "uim/uim-scm.h"
 
 #include "gtk-im-uim.h"
 #include "uim-cand-win-gtk.h"
@@ -82,7 +81,7 @@ extern int compose_handle_key(GdkEventKey *key, IMUIMContext *uic);
 
 struct preedit_segment {
   int attr;
-  char *str;
+  gchar *str;
 };
 
 static int im_uim_fd = -1;
@@ -166,7 +165,7 @@ im_uim_commit_string(void *ptr, const char *str)
   g_signal_emit_by_name(uic, "commit", str);
 
   show_state = uim_scm_symbol_value_bool("bridge-show-input-state?");
-  if (show_state) {
+  if (show_state && uic->win) {
     gdk_window_get_origin(uic->win, &x, &y);
     caret_state_indicator_update(uic->caret_state_indicator, x, y, NULL);
   }
@@ -183,21 +182,23 @@ static gboolean
 get_user_defined_color(PangoColor *color, const gchar *uim_symbol)
 {
   gboolean parsed = FALSE;
-  gchar *literal = uim_symbol_value_str(uim_symbol);
+  char *literal = uim_scm_symbol_value_str(uim_symbol);
 
   if (literal != NULL && literal[0] != '\0')
     parsed = pango_color_parse(color, literal);
 
-  g_free(literal);
+  free(literal);
 
   return parsed;
 }
 
-static char *
-get_preedit_segment(struct preedit_segment *ps, PangoAttrList *attrs, char *str)
+static gchar *
+get_preedit_segment(struct preedit_segment *ps, PangoAttrList *attrs,
+		    gchar *str)
 {
   PangoAttribute *attr;
   const gchar *segment_str = ps->str;
+  gint len;
 
   if ((ps->attr & UPreeditAttr_Separator) && !strcmp(segment_str, ""))
     segment_str = DEFAULT_SEPARATOR_STR;
@@ -259,8 +260,9 @@ get_preedit_segment(struct preedit_segment *ps, PangoAttrList *attrs, char *str)
     }
   }
 
-  str = (char *)realloc(str, strlen(str) + strlen(segment_str) + 1);
-  strcat(str, segment_str);
+  len = strlen(str) + strlen(segment_str) + 1;
+  str = (gchar *)g_realloc(str, len);
+  g_strlcat(str, segment_str, len);
 
   return str;
 }
@@ -303,7 +305,7 @@ show_preedit(GtkIMContext *ic, GtkWidget *preedit_label)
   } else {
     gtk_label_set_text(GTK_LABEL(preedit_label), "");
     gtk_widget_hide(preedit_window);
-    gtk_window_resize(GTK_WINDOW(preedit_window), 0, 0);
+    gtk_window_resize(GTK_WINDOW(preedit_window), 1, 1);
   }
   g_free(str);
   pango_attr_list_unref(attrs);
@@ -565,7 +567,7 @@ clear_cb(void *ptr)
   int i;
 
   for (i = 0; i < uic->nr_psegs; i++)
-    free(uic->pseg[i].str);
+    g_free(uic->pseg[i].str);
   free(uic->pseg);
 
   uic->pseg = NULL;
@@ -1114,7 +1116,7 @@ static void
 im_uim_get_preedit_string(GtkIMContext *ic, gchar **str, PangoAttrList **attrs,
 			  gint *cursor_pos)
 {
-  char *tmp;
+  gchar *tmp;
   int i, pos = 0;
   IMUIMContext *uic = IM_UIM_CONTEXT(ic);
 
@@ -1138,7 +1140,7 @@ im_uim_get_preedit_string(GtkIMContext *ic, gchar **str, PangoAttrList **attrs,
   if (str)
     *str = tmp;
   else
-    free(tmp);
+    g_free(tmp);
 }
 
 static void
