@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2004-2007 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2004-2008 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -31,8 +31,8 @@
 
 */
 
-#ifndef __dict_anthy_word_list_gtk_h_includeded__
-#define __dict_anthy_word_list_gtk_h_includeded__
+#ifndef UIM_DICT_ANTHY_WORD_LIST_GTK_H
+#define UIM_DICT_ANTHY_WORD_LIST_GTK_H
 
 #include <gtk/gtk.h>
 
@@ -79,4 +79,4 @@ GType dict_enum_dictionary_type_get_type (void);
 
 DictEnumDictionaryType dict_get_dictionary_type(GtkWidget *window);
 
-#endif /* __dict_anthy_word_list_gtk_h_includeded__ */
+#endif /* UIM_DICT_ANTHY_WORD_LIST_GTK_H */
