@@ -1,7 +1,7 @@
 /*
  *  $Id:$
  *  Copyright (c) 2003,2004 Masahito Omote <omote@utyuuzin.net>
- *                2005-2007 uim Project http://uim.freedesktop.org/
+ *                2005-2008 uim Project http://code.google.com/p/uim/
  *
  *  All rights reserved.
  *
@@ -39,8 +39,8 @@
  *   * uim-prefѤ˽
  */
 
-#ifndef __dict_word_h_included__
-#define __dict_word_h_included__
+#ifndef UIM_DICT_WORD_H
+#define UIM_DICT_WORD_H
 
 /*
  *  enumǤϤʤʸѹ?
@@ -102,4 +102,4 @@ uim_word *word_last      (uim_word *list);
 
 uim_word_type dict_identifier_to_word_type(char *identifier);
 int dict_identifier_to_support_type(char *identifier);
-#endif /* __dict_word_h_included__ */
+#endif /* UIM_DICT_WORD_H */
