@@ -1,5 +1,5 @@
 ;;;
-;;; Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+;;; Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -52,28 +52,33 @@
   (lambda ()
     (set! prime-cand-select-key?
 	  (lambda (key key-state)
-	    (and (numeral-char? key)
+	    (and (ichar-numeric? key)
 		 (control-key-mask key-state))))))
 
 ;; keys without custom
 (define-key prime-escape-key?         '("escape" "<Control>["))
 (define-key prime-space-key?          '(" "))
 
+(define prime-app-mode-end-stroke-list #f)
 ;;;; If you're a Vi user, modify the lines below.
-(if prime-custom-app-mode-vi?
-    (begin
-      ;; For Vi users
-      (define-key prime-app-mode-start-key? prime-escape-key?)
-      (define prime-app-mode-end-stroke-list
-	'("i" "I" "a" "A" "o" "O" "C" "s" "S" ("c" . ("l" "w" "e" "c" "G")))))
-    (begin
-      ;; Default
-      (define-key prime-app-mode-start-key?  #f)
-      (define prime-app-mode-end-stroke-list #f)))
+(define prime-configure-app-mode-vi
+  (lambda ()
+    (if prime-custom-app-mode-vi?
+	(begin
+	  ;; For Vi users
+	  (define-key prime-app-mode-start-key? prime-escape-key?)
+	  (set! prime-app-mode-end-stroke-list
+		'("i" "I" "a" "A" "o" "O" "C" "s" "S" ("c" . ("l" "w" "e" "c" "G")))))
+	(begin
+	  ;; Default
+	  (define-key prime-app-mode-start-key?  #f)
+	  (set! prime-app-mode-end-stroke-list #f)))))
+
+(prime-configure-app-mode-vi)
 
 (define prime-cand-select-key?
   (lambda (key key-state)
-    (numeral-char? key)))
+    (ichar-numeric? key)))
 
 (define prime-symbol-key?
   (lambda (key key-state)
@@ -101,7 +106,7 @@
 (define prime-capital-key?
   (lambda (key key-state)
     (and (shift-key-mask key-state)
-	 (alphabet-char? key))))
+	 (ichar-alphabetic? key))))
 
 (define prime-ja-direct-key?
   (lambda (key key-state)
@@ -114,7 +119,7 @@
 (define prime-keymap-get-keymap
   (lambda (context key key-state)
     (let ((mode (prime-context-mode context))
-	  (keymap))
+	  (keymap #f))
       (cond 
        ((= mode prime-mode-latin)
 	(set! keymap prime-keymap-latin-mode))
@@ -130,7 +135,7 @@
   (lambda (context key key-state)
     (let ((state    (prime-context-state context))
 	  (language (prime-context-language context))
-	  (keymap))
+	  (keymap #f))
       (cond
        ((eq? state 'prime-state-segment)
 	(set! keymap prime-keymap-segment-state))
@@ -158,9 +163,12 @@
 
 (define prime-keymap-get-command
   (lambda (keymap key key-state)
-    (let ((command-key? (car (car keymap)))
-	  (command      (cdr (car keymap))))
-      (if ((symbol-value command-key?) key key-state)
+    (let* ((command-key? (car (car keymap)))
+	   (command      (cdr (car keymap)))
+	   (key-pred (symbol-value command-key?)))
+      (if (and
+	   key-pred
+	   (key-pred key key-state))
 	  command
 	  (if (null? (cdr keymap))
 	      #f
@@ -231,7 +239,7 @@
   '(
    (prime-wide-latin-key?     . prime-command-wide-latin-mode)
    (prime-latin-key?          . prime-command-latin-mode)
-   (prime-app-mode-start-key? . prime-command-app-mode-start)
+;   (prime-app-mode-start-key? . prime-command-app-mode-start)
    ;; Typing mode key bindings
    (prime-typing-mode-hiragana-key?  . prime-command-mode-hiragana)
    (prime-typing-mode-katakana-key?  . prime-command-mode-katakana)
@@ -540,7 +548,7 @@
 (define prime-context-initialize!
   (lambda (context)
     ;(print "prime-context-initialize!")
-    (if (null? (prime-context-session context))
+    (if (not (prime-context-session context))
 	(begin
 	  ;; The prime server is initialized here.
 	  (prime-lib-init prime-use-unixdomain?)
@@ -560,21 +568,29 @@
   (lambda (context)
     ;(print "prime-context-push")
     (let* ((im (prime-context-im context))
-	   (id (prime-context-id context))
-	   (new-context (prime-context-new2 (prime-context-id context) im)))
+	   (uc (prime-context-uc context))
+           (root-context (im-retrieve-context uc))
+	   (new-context (prime-context-new2 uc im)))
 
       (prime-context-set-history! new-context (prime-context-history context))
       (set-cdr! (assoc 'state (prime-context-history new-context))
 		'prime-state-pushed)
-      (prime-context-set-parent-context! new-context (cons id (cdr context)))
-      (set-cdr! (find-context id) (cdr new-context))
+      (prime-context-set-parent-context! new-context (cons uc (cdr context)))
+      ;; FIXME: Directly overwriting root-context prevents flexible input
+      ;; context composition such as <im-switcher context> -> <prime
+      ;; context>.  In such case, this code break both im-switcher context
+      ;; and prime context. Do not assume and access superordinate input
+      ;; context management scheme. -- YamaKen 2007-01-11
+      (set-cdr! root-context (cdr new-context))
       new-context)))
 
-;; This function destories the current context-data and pops the tail
+;; This function destructs the current context-data and pops the tail
 ;; context-data from the stack of uim-contexts.
 (define prime-context-pop
   (lambda (context)
-    (let ((parent-context (prime-context-parent-context context)))
+    (let* ((parent-context (prime-context-parent-context context))
+           (uc (prime-context-uc context))
+           (root-context (im-retrieve-context uc)))
       (map
        (lambda (lang-pair)
 	 (prime-engine-session-end (cdr lang-pair)))
@@ -585,7 +601,9 @@
  					(prime-context-history parent-context))
 	    (set-cdr! (assoc 'state (prime-context-history context))
 		      'prime-state-poped)
-	    (set-cdr! (find-context (prime-context-id context))
+            ;; FIXME: Directly overwriting root-context. See the comment
+            ;; in prime-context-push.
+	    (set-cdr! root-context
 		      (cdr parent-context))
 	    ))
       parent-context)))
@@ -615,20 +633,22 @@
 			       (prime-context-nth context))))
       (cond
        ((not (equal? state
-		     (cdr (assoc 'state prev-data))))
+		     (safe-cdr (assoc 'state prev-data))))
 	'state)
        ((not (equal? (prime-context-get-preedit-line context)
-		     (cdr (assoc 'preedit-line prev-data))))
+		     (safe-cdr (assoc 'preedit-line prev-data))))
 	'preedit)
        ((not (equal? (prime-context-fund-line context)
-		     (cdr (assoc 'fund-line prev-data))))
+		     (safe-cdr (assoc 'fund-line prev-data))))
 	'cursor)
        ((not (equal? selected-index
-		     (cdr (assoc 'selected-index prev-data))))
+		     (safe-cdr (assoc 'selected-index prev-data))))
 	'nth)
        ((not (equal? (prime-context-modification context)
-		     (cdr (assoc 'conversion-line prev-data))))
+		     (safe-cdr (assoc 'conversion-line prev-data))))
 	'cursor)
+       (else
+        #f)
        ))))
 
 
@@ -670,21 +690,21 @@
     (if (>= index-no (prime-candidates-get-length context))
 	#f
 	(let ((state (prime-context-state context)))
-	  (if (= state 'prime-state-segment)
+	  (if (eq? state 'prime-state-segment)
 	      (car (nth index-no (prime-context-segment-candidates context)))
 	      (car (nth index-no (prime-context-candidates context))))))))
 
 (define prime-candidates-get-length
   (lambda (context)
     (let ((state (prime-context-state context)))
-      (if (= state 'prime-state-segment)
+      (if (eq? state 'prime-state-segment)
 	  (length (prime-context-segment-candidates context))
 	  (length (prime-context-candidates context))))))
 
 (define prime-candidates-get-index
   (lambda (context)
     (let ((state (prime-context-state context)))
-      (if (= state 'prime-state-segment)
+      (if (eq? state 'prime-state-segment)
 	  (prime-context-segment-nth context)
 	  (prime-context-nth context)))))
 
@@ -727,6 +747,12 @@
 ;; (string-split "\t\t" "\t") => ().
 ;; (prime-util-string-split "\t\t" "\t") => ("" "" "").
 ;; The second argument separator must be a single character string.
+;;
+;; uim 1.5.0 revised the specification of string-split as
+;; follows. Replace prime-util-string-split with the new string-split
+;; if no other problems are remaining.  -- YamaKen 2007-07-11
+;;   (string-split "\t\t" "\t")  => ("" "" "")
+;;   (string-split "\t1\t" "\t") => ("" "1" "")
 (define prime-util-string-split
   (lambda (string separator)
     (let ((result (list))
@@ -793,11 +819,13 @@
 ;; problem with unix domain socket.
 (define prime-engine-send-command
   (lambda (arg-list)
-    (cdr 
-     (string-split
-      (prime-send-command
-       (prime-util-string-concat arg-list "\t"))
-      "\n"))))
+    ;; result       ==> "ok\n1\n"
+    ;; result-lines ==> ("ok" "1" "")
+    (let* ((result (prime-send-command
+		    (prime-util-string-concat arg-list "\t")))
+	   (result-lines (string-split result "\n")))
+      (drop-right! result-lines 1) ;; drop last ""
+      (cdr result-lines)))) ;; drop status line
 
 (define prime-engine-conv-predict
   (lambda (prime-session)
@@ -1127,7 +1155,7 @@
 
 (define prime-command-conv-select
   (lambda (context key key-state)
-    (let* ((nth0 (number->candidate-index (numeral-char->number key)))
+    (let* ((nth0 (number->candidate-index (numeric-ichar->integer key)))
 	   (cand-range (prime-uim-candwin-get-range context))
 	   (nth (min (+ (car cand-range) nth0) (cdr cand-range)))
 	   (cand (prime-candidates-get-nth context nth)))
@@ -1139,7 +1167,7 @@
 ;; FIXME: <Hiroyuki Komatsu> (2005-03-30)
 (define prime-command-segment-select
   (lambda (context key key-state)
-    (let* ((nth0 (number->candidate-index (numeral-char->number key)))
+    (let* ((nth0 (number->candidate-index (numeric-ichar->integer key)))
 	   (cand-range (prime-uim-candwin-get-range context))
 	   (nth (min (+ (car cand-range) nth0) (cdr cand-range)))
 	   (cand (prime-candidates-get-nth context nth)))
@@ -1324,7 +1352,7 @@
 (define prime-command-preedit-commit-candidate
   (lambda (context key key-state)
     (if prime-custom-number-selection?
-	(let* ((nth (number->candidate-index (numeral-char->number key)))
+	(let* ((nth (number->candidate-index (numeric-ichar->integer key)))
 	       (cand (prime-get-nth-candidate context nth)))
 	  (if cand
 	      (prime-commit-candidate context nth))
@@ -1451,7 +1479,7 @@
 
 (define prime-command-app-mode-internal
   (lambda (context key key-state key-list)
-    (let ((key-data (car key-list)))
+    (let ((key-data (safe-car key-list)))
       (cond
        ;; there's no speficied command then pressed key is passed.
        ((eq? key-list '())
@@ -1535,17 +1563,21 @@
      ;; right motion
      ((and (> motion-arg 0)
 	   (not (null? (cdr line))))
-      (let ((line-left  (cons (car (prime-editor-get-right line))
+      (let ((line-left  (cons (or (safe-car (prime-editor-get-right line))
+				  '())
 			      (prime-editor-get-left line)))
-	    (line-right (cdr (prime-editor-get-right line))))
+	    (line-right (or (safe-cdr (prime-editor-get-right line))
+			    '())))
 	(prime-editor-set-left  line line-left)
 	(prime-editor-set-right line line-right))
       (prime-editor-cursor-move line (- motion-arg 1)))
      ;; left motion
      ((and (< motion-arg 0)
 	   (not (null? (car line))))
-      (let ((line-left  (cdr (prime-editor-get-left line)))
-	    (line-right (cons (car (prime-editor-get-left line))
+      (let ((line-left  (or (safe-cdr (prime-editor-get-left line))
+			    '()))
+	    (line-right (cons (or (safe-car (prime-editor-get-left line))
+				  '())
 			      (prime-editor-get-right line))))
 	(prime-editor-set-left  line line-left)
 	(prime-editor-set-right line line-right))
@@ -1558,11 +1590,13 @@
 
 (define prime-editor-backspace-char
   (lambda (line)
-    (prime-editor-set-left  line (cdr (prime-editor-get-left line)))))
+    (prime-editor-set-left  line (or (safe-cdr (prime-editor-get-left line))
+				     '()))))
 
 (define prime-editor-delete-char
   (lambda (line)
-    (prime-editor-set-right line (cdr (prime-editor-get-right line)))))
+    (prime-editor-set-right line (or (safe-cdr (prime-editor-get-right line))
+				     '()))))
 
 
 ;; This returns a preediting string.
@@ -1647,9 +1681,12 @@
     (prime-learn-word context word-data)
     (prime-commit-string
      context
-     (string-append (or (cadr (assoc "base"        word-data)) "")
-		    (or (cadr (assoc "conjugation" word-data)) "")
-		    (or (cadr (assoc "suffix"      word-data)) "")))))
+     (string-append (or (safe-car (safe-cdr (assoc "base"        word-data)))
+			"")
+		    (or (safe-car (safe-cdr (assoc "conjugation" word-data)))
+			"")
+		    (or (safe-car (safe-cdr (assoc "suffix"      word-data)))
+			"")))))
 
 (define prime-commit-preedition
   (lambda (context)
@@ -1689,12 +1726,17 @@
 
 (define prime-learn-word
   (lambda (context assoc-list)
-    (let ((key     (or (cadr (assoc "basekey"     assoc-list)) ""))
-	  (value   (or (cadr (assoc "base"        assoc-list)) ""))
-	  (part    (or (cadr (assoc "part"        assoc-list)) ""))
+    (let ((key     (or (safe-car (safe-cdr (assoc "basekey"     assoc-list)))
+		       ""))
+	  (value   (or (safe-car (safe-cdr (assoc "base"        assoc-list)))
+		       ""))
+	  (part    (or (safe-car (safe-cdr (assoc "part"        assoc-list)))
+		       ""))
 	  (prime-context (or (prime-context-last-word context) ""))
-	  (suffix  (or (cadr (assoc "conjugation" assoc-list)) ""))
-	  (rest    (or (cadr (assoc "suffix"      assoc-list)) "")))
+	  (suffix  (or (safe-car (safe-cdr (assoc "conjugation" assoc-list)))
+		       ""))
+	  (rest    (or (safe-car (safe-cdr (assoc "suffix"      assoc-list)))
+		       "")))
       
       (prime-engine-learn-word key value part prime-context suffix rest)
       (prime-context-set-last-word! context
@@ -1718,7 +1760,7 @@
 
 (define prime-convert-start-internal
   (lambda (context init-idx)
-    (let ((res))
+    (let ((res #f))
       (prime-convert-get-conversion context)
       (set! res (prime-get-nth-candidate context init-idx))
       (if res
@@ -1781,7 +1823,7 @@
     (let ((session (prime-context-session context))
 	  (mode (prime-context-mode context)))
       (cond
-       ((null? session)
+       ((not session)
 	#f)  ;; Do nothing.
 
        (else
@@ -1801,7 +1843,7 @@
   (lambda (context)
     (let ((session (prime-context-session context)))
       (cond
-       ((null? session)
+       ((not session)
 	#f)  ;; Do nothing.
 
        (else
@@ -1950,7 +1992,7 @@
 
 (define prime-display-preedit
   (lambda (context preedit-list)
-    (if preedit-list
+    (if (not (null? preedit-list))
 	(let ((type   (car (car preedit-list)))
 	      (string (cdr (car preedit-list))))
 	  (cond
@@ -2026,7 +2068,7 @@
 
 (define prime-press-key-handler
   (lambda (context key key-state)
-    (if (control-char? key)
+    (if (ichar-control? key)
 	(im-commit-raw context)
 	(let ((keymap (prime-keymap-get-keymap context key key-state)))
 	  (prime-proc-call-command keymap context key key-state)
@@ -2036,9 +2078,9 @@
 (define prime-release-key-handler
   (lambda (context key key-state)
     ;(print "prime-release-key-handler")
-    (if (or (control-char? key)
-	    (= (prime-context-mode context)
-	       prime-mode-latin))
+    (if (or (ichar-control? key)
+	    (= (prime-context-mode context) prime-mode-latin)
+	    (= (prime-context-mode context) prime-mode-application))
 	(im-commit-raw context)
 	;; else
 	;; FIXME: update candidate words.
@@ -2069,7 +2111,7 @@
 (define prime-mode-language-set
   (lambda (context language)
     (let* ((lang-session-list (prime-context-lang-session-list context))
-	   (session (cdr (assoc language lang-session-list))))
+	   (session (safe-cdr (assoc language lang-session-list))))
       (if (not session)
 	  (begin
 	    (set! session (prime-engine-session-language-set language))
@@ -2087,7 +2129,8 @@
 	       (nth index-no (prime-context-candidates context)))))
       ;; The return value is a list with a candidate string and the next index.
       (list (prime-candidate-combine-string context candidate)
-	    (digit->string (+ index-no 1))))))
+	    (digit->string (+ index-no 1))
+	    ""))))
 
 (define prime-candidate-combine-string
   (lambda (context candidate)
@@ -2119,7 +2162,7 @@
 
 (define prime-candidate-get-data
   (lambda (candidate key)
-    (cadr (assoc key (nth 1 candidate)))))
+    (safe-car (safe-cdr (assoc key (nth 1 candidate))))))
 
 (define prime-set-candidate-index-handler
   (lambda (context selection-index)
