@@ -1,5 +1,5 @@
 ;;;
-;;; Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+;;; Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -27,7 +27,7 @@
 ;;; OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 ;;; SUCH DAMAGE.
 ;;;;
-;; 2006-11-13 version
+;; 2007-12-10 version
 
 ;;
 (require "generic.scm")
@@ -152,7 +152,7 @@
     ((("m" "3"))("")) (((" " "m" "3"))(""))
     ((("w" "3"))("")) (((" " "w" "3"))(""))
     ((("v" "3"))("")) (((" " "v" "3"))(""))
-    ((("z" "3"))("Ĭ")) (((" " "z" "3"))(""))
+    ((("z" "3"))("")) (((" " "z" "3"))(""))
     ((("/" "3"))("")) (((" " "/" "3"))(""))
     ((("1" "4"))("")) (((" " "1" "4"))(""))
     ((("2" "4"))("")) (((" " "2" "4"))(""))
@@ -193,17 +193,17 @@
     ((("w" "4"))("")) (((" " "w" "4"))(""))
     ((("v" "4"))("")) (((" " "v" "4"))(""))
     ((("z" "4"))("Ĭ")) (((" " "z" "4"))(""))
-    ((("/" "4"))(""))  (((" " "/" "4"))(""))
+    ((("/" "4"))("")) (((" " "/" "4"))(""))
     ((("1" "5"))("")) (((" " "1" "5"))(""))
-    ((("2" "5"))("")) (((" " "1" "5"))(""))
-    ((("3" "5"))("")) (((" " "1" "5"))(""))
-    ((("4" "5"))("")) (((" " "1" "5"))(""))
-    ((("5" "5"))("")) (((" " "1" "5"))(""))
-    ((("6" "5"))("")) (((" " "1" "5"))(""))
-    ((("7" "5"))("")) (((" " "1" "5"))(""))
-    ((("8" "5"))("")) (((" " "1" "5"))(""))
-    ((("9" "5"))("")) (((" " "1" "5"))(""))
-    ((("0" "5"))("")) (((" " "1" "5"))(""))
+    ((("2" "5"))("")) (((" " "2" "5"))(""))
+    ((("3" "5"))("")) (((" " "3" "5"))(""))
+    ((("4" "5"))("")) (((" " "4" "5"))(""))
+    ((("5" "5"))("")) (((" " "5" "5"))(""))
+    ((("6" "5"))("")) (((" " "6" "5"))(""))
+    ((("7" "5"))("")) (((" " "7" "5"))(""))
+    ((("8" "5"))("")) (((" " "8" "5"))(""))
+    ((("9" "5"))("")) (((" " "9" "5"))(""))
+    ((("0" "5"))("")) (((" " "0" "5"))(""))
     ((("," "5"))("")) (((" " "," "5"))(""))
     ((("." "5"))("")) (((" " "." "5"))(""))
     ((("p" "5"))("")) (((" " "p" "5"))(""))
@@ -234,86 +234,86 @@
     ((("v" "5"))("")) (((" " "v" "5"))(""))
     ((("z" "5"))("")) (((" " "z" "5"))(""))
     ((("/" "5"))("")) (((" " "/" "5"))(""))
-    ((("1" "6"))("")) (((" ""1" "6"))(""))
-    ((("2" "6"))("")) (((" ""1" "6"))(""))
-    ((("3" "6"))("")) (((" ""1" "6"))(""))
-    ((("4" "6"))("")) (((" ""1" "6"))(""))
-    ((("5" "6"))("")) (((" ""1" "6"))(""))
-    ((("6" "6"))("")) (((" ""1" "6"))(""))
-    ((("7" "6"))("")) (((" ""1" "6"))(""))
-    ((("8" "6"))("")) (((" ""1" "6"))(""))
-    ((("9" "6"))("")) (((" ""1" "6"))(""))
-    ((("0" "6"))("")) (((" ""1" "6"))(""))
-    ((("," "6"))("")) (((" ""1" "6"))(""))
-    ((("." "6"))("")) (((" ""1" "6"))("é"))
-    ((("p" "6"))("")) (((" ""1" "6"))(""))
-    ((("y" "6"))("©")) (((" ""1" "6"))(""))
-    ((("f" "6"))("")) (((" ""1" "6"))(""))
-    ((("g" "6"))("")) (((" ""1" "6"))(""))
-    ((("c" "6"))("")) (((" ""1" "6"))(""))
-    ((("r" "6"))("")) (((" ""1" "6"))(""))
-    ((("l" "6"))("Ƹ")) (((" ""1" "6"))(""))
-    ((("a" "6"))("")) (((" ""1" "6"))(""))
-    ((("o" "6"))("")) (((" ""1" "6"))(""))
-    ((("e" "6"))("")) (((" ""1" "6"))(""))
-    ((("u" "6"))("")) (((" ""1" "6"))(""))
-    ((("i" "6"))("")) (((" ""1" "6"))(""))
-    ((("d" "6"))("")) (((" ""1" "6"))(""))
-    ((("h" "6"))("")) (((" ""1" "6"))(""))
-    ((("t" "6"))("")) (((" ""1" "6"))(""))
-    ((("n" "6"))("")) (((" ""1" "6"))("Ŀ"))
-    ((("s" "6"))("")) (((" ""1" "6"))(""))
-    (((";" "6"))("Ȫ")) (((" ""1" "6"))(""))
-    ((("q" "6"))("")) (((" ""1" "6"))(""))
-    ((("j" "6"))("")) (((" ""1" "6"))(""))
-    ((("k" "6"))("ū")) (((" ""1" "6"))(""))
-    ((("x" "6"))("")) (((" ""1" "6"))(""))
-    ((("b" "6"))("")) (((" ""1" "6"))(""))
-    ((("m" "6"))("")) (((" ""1" "6"))(""))
-    ((("w" "6"))("")) (((" ""1" "6"))(""))
-    ((("v" "6"))("")) (((" ""1" "6"))(""))
-    ((("z" "6"))("")) (((" ""1" "6"))(""))
-    ((("/" "6"))("")) (((" ""1" "6"))(""))
-    ((("1" "7"))("")) (((" ""1" "7"))(""))
-    ((("2" "7"))("")) (((" ""2" "7"))(""))
-    ((("3" "7"))("")) (((" ""3" "7"))(""))
-    ((("4" "7"))("")) (((" ""4" "7"))(""))
-    ((("5" "7"))("")) (((" ""5" "7"))(""))
-    ((("6" "7"))("")) (((" ""6" "7"))(""))
-    ((("7" "7"))("")) (((" ""7" "7"))(""))
-    ((("8" "7"))("")) (((" ""8" "7"))(""))
-    ((("9" "7"))("")) (((" ""9" "7"))(""))
-    ((("0" "7"))("")) (((" ""0" "7"))(""))
-    ((("," "7"))("»")) (((" ""," "7"))(""))
-    ((("." "7"))("ɶ")) (((" ""." "7"))(""))
-    ((("p" "7"))("")) (((" ""p" "7"))(""))
-    ((("y" "7"))("")) (((" ""y" "7"))(""))
-    ((("f" "7"))("")) (((" ""f" "7"))(""))
-    ((("g" "7"))("æ")) (((" ""g" "7"))(""))
-    ((("c" "7"))("")) (((" ""c" "7"))(""))
-    ((("r" "7"))("")) (((" ""r" "7"))(""))
-    ((("l" "7"))("º")) (((" ""l" "7"))(""))
-    ((("a" "7"))("")) (((" ""a" "7"))(""))
-    ((("o" "7"))("")) (((" ""o" "7"))(""))
-    ((("e" "7"))("͹")) (((" ""e" "7"))(""))
-    ((("u" "7"))("")) (((" ""u" "7"))(""))
-    ((("i" "7"))("")) (((" ""i" "7"))(""))
-    ((("d" "7"))("")) (((" ""d" "7"))(""))
-    ((("h" "7"))("")) (((" ""h" "7"))(""))
-    ((("t" "7"))("")) (((" ""t" "7"))(""))
-    ((("n" "7"))("")) (((" ""n" "7"))(""))
-    ((("s" "7"))("")) (((" ""s" "7"))(""))
-    (((";" "7"))("")) (((" "";" "7"))(""))
-    ((("q" "7"))("ü")) (((" ""q" "7"))(""))
-    ((("j" "7"))("")) (((" ""j" "7"))(""))
-    ((("k" "7"))("")) (((" ""k" "7"))(""))
-    ((("x" "7"))("")) (((" ""x" "7"))(""))
-    ((("b" "7"))("")) (((" ""b" "7"))(""))
-    ((("m" "7"))("")) (((" ""m" "7"))(""))
-    ((("w" "7"))("")) (((" ""w" "7"))(""))
-    ((("v" "7"))("")) (((" ""v" "7"))(""))
-    ((("z" "7"))("")) (((" ""z" "7"))(""))
-    ((("/" "7"))("")) (((" ""/" "7"))("м"))
+    ((("1" "6"))("")) (((" " "1" "6"))(""))
+    ((("2" "6"))("")) (((" " "2" "6"))(""))
+    ((("3" "6"))("")) (((" " "3" "6"))(""))
+    ((("4" "6"))("")) (((" " "4" "6"))(""))
+    ((("5" "6"))("")) (((" " "5" "6"))(""))
+    ((("6" "6"))("")) (((" " "6" "6"))(""))
+    ((("7" "6"))("")) (((" " "7" "6"))(""))
+    ((("8" "6"))("")) (((" " "8" "6"))(""))
+    ((("9" "6"))("")) (((" " "9" "6"))(""))
+    ((("0" "6"))("")) (((" " "0" "6"))(""))
+    ((("," "6"))("")) (((" " "," "6"))(""))
+    ((("." "6"))("")) (((" " "." "6"))("é"))
+    ((("p" "6"))("")) (((" " "p" "6"))(""))
+    ((("y" "6"))("©")) (((" " "y" "6"))(""))
+    ((("f" "6"))("")) (((" " "f" "6"))(""))
+    ((("g" "6"))("")) (((" " "g" "6"))(""))
+    ((("c" "6"))("")) (((" " "c" "6"))(""))
+    ((("r" "6"))("")) (((" " "r" "6"))(""))
+    ((("l" "6"))("Ƹ")) (((" " "l" "6"))(""))
+    ((("a" "6"))("")) (((" " "a" "6"))(""))
+    ((("o" "6"))("")) (((" " "o" "6"))(""))
+    ((("e" "6"))("")) (((" " "e" "6"))(""))
+    ((("u" "6"))("")) (((" " "u" "6"))(""))
+    ((("i" "6"))("")) (((" " "i" "6"))(""))
+    ((("d" "6"))("")) (((" " "d" "6"))(""))
+    ((("h" "6"))("")) (((" " "h" "6"))(""))
+    ((("t" "6"))("")) (((" " "t" "6"))(""))
+    ((("n" "6"))("")) (((" " "n" "6"))("Ŀ"))
+    ((("s" "6"))("")) (((" " "s" "6"))(""))
+    (((";" "6"))("Ȫ")) (((" " ";" "6"))(""))
+    ((("q" "6"))("")) (((" " "q" "6"))(""))
+    ((("j" "6"))("")) (((" " "j" "6"))(""))
+    ((("k" "6"))("ū")) (((" " "k" "6"))(""))
+    ((("x" "6"))("")) (((" " "x" "6"))(""))
+    ((("b" "6"))("")) (((" " "b" "6"))(""))
+    ((("m" "6"))("")) (((" " "m" "6"))(""))
+    ((("w" "6"))("")) (((" " "w" "6"))(""))
+    ((("v" "6"))("")) (((" " "v" "6"))(""))
+    ((("z" "6"))("")) (((" " "z" "6"))(""))
+    ((("/" "6"))("")) (((" " "/" "6"))(""))
+    ((("1" "7"))("")) (((" " "1" "7"))(""))
+    ((("2" "7"))("")) (((" " "2" "7"))(""))
+    ((("3" "7"))("")) (((" " "3" "7"))(""))
+    ((("4" "7"))("")) (((" " "4" "7"))(""))
+    ((("5" "7"))("")) (((" " "5" "7"))(""))
+    ((("6" "7"))("")) (((" " "6" "7"))(""))
+    ((("7" "7"))("")) (((" " "7" "7"))(""))
+    ((("8" "7"))("")) (((" " "8" "7"))(""))
+    ((("9" "7"))("")) (((" " "9" "7"))(""))
+    ((("0" "7"))("")) (((" " "0" "7"))(""))
+    ((("," "7"))("»")) (((" " "," "7"))(""))
+    ((("." "7"))("ɶ")) (((" " "." "7"))(""))
+    ((("p" "7"))("")) (((" " "p" "7"))(""))
+    ((("y" "7"))("")) (((" " "y" "7"))(""))
+    ((("f" "7"))("")) (((" " "f" "7"))(""))
+    ((("g" "7"))("æ")) (((" " "g" "7"))(""))
+    ((("c" "7"))("")) (((" " "c" "7"))(""))
+    ((("r" "7"))("")) (((" " "r" "7"))(""))
+    ((("l" "7"))("º")) (((" " "l" "7"))(""))
+    ((("a" "7"))("")) (((" " "a" "7"))(""))
+    ((("o" "7"))("")) (((" " "o" "7"))(""))
+    ((("e" "7"))("͹")) (((" " "e" "7"))(""))
+    ((("u" "7"))("")) (((" " "u" "7"))(""))
+    ((("i" "7"))("")) (((" " "i" "7"))(""))
+    ((("d" "7"))("")) (((" " "d" "7"))(""))
+    ((("h" "7"))("")) (((" " "h" "7"))(""))
+    ((("t" "7"))("")) (((" " "t" "7"))(""))
+    ((("n" "7"))("")) (((" " "n" "7"))(""))
+    ((("s" "7"))("")) (((" " "s" "7"))(""))
+    (((";" "7"))("")) (((" " ";" "7"))(""))
+    ((("q" "7"))("ü")) (((" " "q" "7"))(""))
+    ((("j" "7"))("")) (((" " "j" "7"))(""))
+    ((("k" "7"))("")) (((" " "k" "7"))(""))
+    ((("x" "7"))("")) (((" " "x" "7"))(""))
+    ((("b" "7"))("")) (((" " "b" "7"))(""))
+    ((("m" "7"))("")) (((" " "m" "7"))(""))
+    ((("w" "7"))("")) (((" " "w" "7"))(""))
+    ((("v" "7"))("")) (((" " "v" "7"))(""))
+    ((("z" "7"))("")) (((" " "z" "7"))(""))
+    ((("/" "7"))("")) (((" " "/" "7"))("м"))
     ((("1" "8"))("")) (((" " "1" "8"))(""))
     ((("2" "8"))("")) (((" " "2" "8"))(""))
     ((("3" "8"))("")) (((" " "3" "8"))(""))
@@ -366,7 +366,7 @@
     ((("0" "9"))("")) (((" " "0" "9"))(""))
     ((("," "9"))("Ƽ")) (((" " "," "9"))(""))
     ((("." "9"))("")) (((" " "." "9"))("ʯ"))
-    ((("p" "9"))("")) (((" " "p" "9"))(""))
+    ((("p" "9"))("Ģ")) (((" " "p" "9"))(""))
     ((("y" "9"))("")) (((" " "y" "9"))(""))
     ((("f" "9"))("")) (((" " "f" "9"))("ʺ"))
     ((("g" "9"))("")) (((" " "g" "9"))(""))
@@ -385,13 +385,13 @@
     ((("s" "9"))("")) (((" " "s" "9"))(""))
     (((";" "9"))("")) (((" " ";" "9"))(""))
     ((("q" "9"))("")) (((" " "q" "9"))(""))
-    ((("j" "9"))("")) (((" " "i" "9"))(""))
+    ((("j" "9"))("")) (((" " "j" "9"))(""))
     ((("k" "9"))("")) (((" " "k" "9"))(""))
     ((("x" "9"))("")) (((" " "x" "9"))(""))
     ((("b" "9"))("")) (((" " "b" "9"))(""))
     ((("m" "9"))("")) (((" " "m" "9"))(""))
     ((("w" "9"))("")) (((" " "w" "9"))(""))
-    ((("v" "9"))("ð")) (((" " "v" "9"))(""))
+    ((("v" "9"))("ð")) (((" " "v" "9"))("ó"))
     ((("z" "9"))("")) (((" " "z" "9"))(""))
     ((("/" "9"))("")) (((" " "/" "9"))(""))
     ((("1" "0"))("")) (((" " "1" "0"))(""))
@@ -576,7 +576,7 @@
     ((("a" "y"))("")) (((" " "a" "y"))(""))
     ((("o" "y"))("Ϻ")) (((" " "o" "y"))("ϭ"))
     ((("e" "y"))("")) (((" " "e" "y"))(""))
-    ((("u" "y"))("ˡ")) (((" " "y" "y"))(""))
+    ((("u" "y"))("ˡ")) (((" " "u" "y"))(""))
     ((("i" "y"))("")) (((" " "i" "y"))(""))
     ((("d" "y"))("Ǥ")) (((" " "d" "y"))(""))
     ((("h" "y"))("")) (((" " "h" "y"))(""))
@@ -619,12 +619,12 @@
     ((("u" "f"))("")) (((" " "u" "f"))(""))
     ((("i" "f"))("6")) (((" " "i" "f"))(""))
     ((("d" "f"))("")) (((" " "d" "f"))(""))
-    ((("h" "f"))("")) (((" " "h" "f"))("."))
+    ((("h" "f"))("")) (((" " "h" "f"))(""))
     ((("t" "f"))("")) (((" " "t" "f"))(""))
     ((("n" "f"))("")) (((" " "n" "f"))(""))
     ((("s" "f"))("ͽ")) (((" " "s" "f"))("ͱ"))
     (((";" "f"))("2")) (((" " ";" "f"))(""))
-    ((("q" "f"))("")) (((" " "g" "f"))(""))
+    ((("q" "f"))("")) (((" " "q" "f"))(""))
     ((("j" "f"))("")) (((" " "j" "f"))(""))
     ((("k" "f"))("0")) (((" " "k" "f"))(""))
     ((("x" "f"))("")) (((" " "x" "f"))(""))
@@ -786,7 +786,7 @@
     (((";" "l"))("ʸ")) (((" " ";" "l"))(""))
     ((("q" "l"))("")) (((" " "q" "l"))(""))
     ((("j" "l"))("")) (((" " "j" "l"))("¥"))
-    ((("k" "l"))("ŷ")) (((" " "k" "l"))("ŷ"))
+    ((("k" "l"))("ŷ")) (((" " "k" "l"))(""))
     ((("x" "l"))("")) (((" " "x" "l"))(""))
     ((("b" "l"))("")) (((" " "b" "l"))("ķ"))
     ((("m" "l"))("")) (((" " "m" "l"))(""))
@@ -1025,7 +1025,7 @@
     ((("s" "d"))("")) (((" " "s" "d"))(""))
     (((";" "d"))("1")) (((" " ";" "d"))(""))
     ((("q" "d"))("")) (((" " "q" "d"))(""))
-    ((("j" "d"))("")) (((" " "j" "d"))(","))
+    ((("j" "d"))("")) (((" " "j" "d"))(""))
     ((("k" "d"))("")) (((" " "k" "d"))(""))
     ((("x" "d"))("")) (((" " "x" "d"))(""))
     ((("b" "d"))("")) (((" " "b" "d"))(""))
@@ -1107,8 +1107,8 @@
     ((("q" "t"))("")) (((" " "q" "t"))(""))
     ((("j" "t"))("")) (((" " "j" "t"))(""))
     ((("k" "t"))("8")) (((" " "k" "t"))("˪"))
-    ((("x" "t"))("")) (((" " "" "t"))(""))
-    ((("b" "t"))("")) (((" " "x" "t"))(""))
+    ((("x" "t"))("")) (((" " "x" "t"))(""))
+    ((("b" "t"))("")) (((" " "b" "t"))(""))
     ((("m" "t"))("")) (((" " "m" "t"))(""))
     ((("w" "t"))("")) (((" " "w" "t"))(""))
     ((("v" "t"))("")) (((" " "v" "t"))(""))
