@@ -2,7 +2,7 @@
 
   uim-ipc.c: Utility functions for inter process communication.
 
-  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -44,7 +44,6 @@
 #include <string.h>
 
 #include "uim-internal.h"
-#include "uim-helper.h"
 #include "uim-util.h"
 
 /* This function is come from the GNU C Library manual */
@@ -70,6 +69,7 @@ open_pipe_rw(FILE **fr, FILE **fw)
   int pipe_fd[2];
   pid_t pid;
   int res;
+  ssize_t nr;
 
   if (pipe(pipe_fd) < 0)
     return (pid_t)-1;
@@ -121,7 +121,9 @@ open_pipe_rw(FILE **fr, FILE **fw)
       *fw = fdopen(fdw[1], "w");
   }
 
-  read(pipe_fd[0], &pid, sizeof(pid_t));
+  if ((nr = read(pipe_fd[0], &pid, sizeof(pid_t))) == -1 || nr == 0)
+    goto err0;
+
   close(pipe_fd[0]);
   close(pipe_fd[1]);
 
@@ -142,11 +144,13 @@ open_pipe_rw(FILE **fr, FILE **fw)
   return (pid_t) -1;
 }
 
-int
-uim_ipc_open_command_with_option(int old_pid, FILE **read_fp,
-		     FILE **write_fp, const char *command, const char *option)
+pid_t
+uim_ipc_open_command_with_option(pid_t old_pid,
+				 FILE **read_fp, FILE **write_fp,
+				 const char *command, const char *option)
 {
-  int new_pid, result;
+  int result;
+  pid_t new_pid;
   char **ap, *argv[10];
   char *str = NULL, *p;
 
@@ -185,7 +189,7 @@ uim_ipc_open_command_with_option(int old_pid, FILE **read_fp,
       argv[1] = NULL;
     } else {
       argv[0] = (char *)command;
-      str = p = strdup(option);
+      str = p = uim_strdup(option);
       for (ap = &argv[1]; (*ap = strsep(&p, " ")) != NULL;) {
 	if (**ap != '\0')
 	  if (++ap >= &argv[9])
@@ -195,7 +199,7 @@ uim_ipc_open_command_with_option(int old_pid, FILE **read_fp,
     }
     if (uim_issetugid()) {
       int cmd_len = strlen(command) + 30;
-      char *fullpath_command = malloc(cmd_len);
+      char *fullpath_command = uim_malloc(cmd_len);
       char *cmd_name = strrchr(command, '/');
 
       if (cmd_name && cmd_name + 1 != '\0')
@@ -231,19 +235,20 @@ uim_ipc_open_command_with_option(int old_pid, FILE **read_fp,
   return new_pid;
 }
 
-int
-uim_ipc_open_command(int old_pid, FILE **read_fp,
-		     FILE **write_fp, const char *command)
+pid_t
+uim_ipc_open_command(pid_t old_pid,
+		     FILE **read_fp, FILE **write_fp, const char *command)
 {
-  return uim_ipc_open_command_with_option(old_pid, read_fp, write_fp, command, NULL);
+  return uim_ipc_open_command_with_option(old_pid,
+					  read_fp, write_fp, command, NULL);
 }
 
 char *
-uim_ipc_send_command(int *pid,
+uim_ipc_send_command(pid_t *pid,
 		     FILE **read_fp, FILE **write_fp,
 		     const char *command, const char *str)
 {
-  char *tmp = strdup("");
+  char *tmp = uim_strdup("");
   char buf[8192];
 
   if (*read_fp == NULL || *write_fp == NULL) {
@@ -280,7 +285,7 @@ uim_ipc_send_command(int *pid,
       break;
     }
 
-    tmp = realloc(tmp, strlen(tmp) + strlen(buf) + 1);
+    tmp = uim_realloc(tmp, strlen(tmp) + strlen(buf) + 1);
     strcat(tmp, buf);
   }
  
@@ -291,19 +296,3 @@ uim_ipc_send_command(int *pid,
   *pid = uim_ipc_open_command(*pid, read_fp, write_fp, command);
   return NULL;
 }
-
-int
-uim_helper_check_connection_fd(int fd)
-{
-  uid_t euid;
-  gid_t egid;
-  if (getpeereid(fd, &euid, &egid) < 0) {
-    perror("getpeereid failed");
-    return -1;
-  }
-  if ((euid != 0) && (euid != getuid())) {
-    fprintf(stderr, "uid mismatch\n");
-    return -1;
-  }
-  return 0;
-}
