@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -30,8 +30,8 @@
   SUCH DAMAGE.
 */
 
-#ifndef _connection_h_included_
-#define _connection_h_included_
+#ifndef UIM_XIM_CONNECTION_H
+#define UIM_XIM_CONNECTION_H
 
 #include "xim.h"
 #include "xdispatch.h"
