@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -37,6 +37,7 @@
 #endif
 
 #include <stdio.h>
+#include <string.h>
 #include <locale.h>
 #include <iconv.h>
 #include <errno.h>
@@ -235,19 +236,22 @@ private:
 UTF8_Locale::UTF8_Locale(const char *encoding)
 {
     mEncoding = strdup(encoding);
-    m_iconv_cd = (iconv_t)uim_iconv_open(encoding, "UTF-8");
+    if (uim_iconv->is_convertible(encoding, "UTF-8"))
+	m_iconv_cd = (iconv_t)uim_iconv->create(encoding, "UTF-8");
+    else
+	m_iconv_cd = (iconv_t)-1;
 }
 
 UTF8_Locale::~UTF8_Locale()
 {
     free(mEncoding);
-    if (m_iconv_cd != (iconv_t)-1)
-        iconv_close(m_iconv_cd);
+    if (m_iconv_cd != (iconv_t)-1 && m_iconv_cd)
+        uim_iconv->release(m_iconv_cd);
 }
 
 char *UTF8_Locale::utf8_to_native_str(char *utf8) {
     size_t outbufsize = BUFSIZ;
-    char *inbuf, *outbuf, *convstr = NULL;
+    char *inbuf, outbuf[BUFSIZ], *convstr = NULL;
     char *inchar;
     char *outchar;
     size_t inbytesleft, outbytesleft;
@@ -260,30 +264,33 @@ char *UTF8_Locale::utf8_to_native_str(char *utf8) {
     if (!inbuf)
 	return NULL;
 
-    outbuf = (char *)malloc(outbufsize);
-    if (!outbuf)
-	return NULL;
+    // no conversion needed
+    if (!m_iconv_cd)
+	return strdup(inbuf);
 
     inchar = inbuf;
     outchar = outbuf;
     inbytesleft = strlen(inbuf);
-    outbytesleft = outbufsize;
+    outbytesleft = outbufsize - 1;
     ret_val = iconv(m_iconv_cd, (ICONV_CONST char **)&inchar, &inbytesleft, &outchar, &outbytesleft);
-
     if (ret_val == (size_t)-1 && errno != E2BIG) {
 	//perror("error in iconv");
-	free(outbuf);
+	return NULL;
+    }
+    ret_val = iconv(m_iconv_cd, NULL, NULL, &outchar, &outbytesleft);
+    if (ret_val == (size_t)-1 && errno != E2BIG) {
+	//perror("error in iconv");
 	return NULL;
     }
 
-    convstr = (char *)malloc(outbufsize - outbytesleft + 1);
+    convstr = (char *)malloc(outbufsize - outbytesleft);
     if (!convstr) {
-	free(outbuf);
 	return NULL;
     }
 
-    strlcpy(convstr, outbuf, outbufsize - outbytesleft + 1);
-    free(outbuf);
+    *outchar = '\0';
+
+    strlcpy(convstr, outbuf, outbufsize - outbytesleft);
     return convstr;
 }
 
@@ -590,22 +597,22 @@ utf8_wctomb(unsigned char *dest, uchar wc)
 	return RET_ILSEQ;
     switch (count) { // note: falls through cases (no break)
     case 6:
-	dest[5] = 0x80 | (wc & 0x3f);
+	dest[5] = (unsigned char)(0x80 | (wc & 0x3f));
 	wc = wc >> 6; wc |= 0x4000000;
     case 5:
-	dest[4] = 0x80 | (wc & 0x3f);
+	dest[4] = (unsigned char)(0x80 | (wc & 0x3f));
 	wc = wc >> 6; wc |= 0x200000;
     case 4:
-	dest[3] = 0x80 | (wc & 0x3f);
+	dest[3] = (unsigned char)(0x80 | (wc & 0x3f));
 	wc = wc >> 6; wc |= 0x10000;
     case 3:
-	dest[2] = 0x80 | (wc & 0x3f);
+	dest[2] = (unsigned char)(0x80 | (wc & 0x3f));
 	wc = wc >> 6; wc |= 0x800;
     case 2:
-	dest[1] = 0x80 | (wc & 0x3f);
+	dest[1] = (unsigned char)(0x80 | (wc & 0x3f));
 	wc = wc >> 6; wc |= 0xc0;
     case 1:
-	dest[0] = wc;
+	dest[0] = (unsigned char)wc;
     }
     return count;
 }
