@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -34,6 +34,8 @@
 # include <config.h>
 #endif
 
+#define UIM_XIM_USE_JAPANESE_KANA_KEYBOARD_HACK 1
+
 #include <stdio.h>
 #include <ctype.h>
 #include <locale.h>
@@ -49,9 +51,6 @@
 #define XK_KATAKANA
 #endif
 #include <X11/keysymdef.h>
-#ifndef XK_dead_horn
-#define XK_dead_horn	0xfe62
-#endif
 
 #include "xim.h"
 #include "convdisp.h"
@@ -62,8 +61,14 @@
 
 #include "uim/uim-helper.h"
 #include "uim/uim-im-switcher.h"
-#include "uim/uim-compat-scm.h"
-#include "uim/uim-stdint.h"
+#include "uim/uim-scm.h"
+#if UIM_XIM_USE_JAPANESE_KANA_KEYBOARD_HACK
+#include "uim/uim-x-util.h"
+#endif
+
+#ifndef XK_dead_horn
+#define XK_dead_horn	0xfe62
+#endif
 
 #ifndef __GNUC__
 # ifdef HAVE_ALLOCA_H
@@ -154,7 +159,7 @@ void XimServer::customContext(const char *custom, const char *val) {
 
     // Updated global IM of XimServer
     if (!strcmp(custom, "custom-preserved-default-im-name") &&
-		    uim_scm_symbol_value_bool("custom-activate-default-im-name?"))
+	uim_scm_symbol_value_bool("custom-activate-default-im-name?"))
 	set_im(++val);
 
 #if HAVE_XFT_UTF8_STRING
@@ -163,7 +168,7 @@ void XimServer::customContext(const char *custom, const char *val) {
 #endif
 
     if (!strcmp(custom, "bridge-show-input-state?") &&
-		    !uim_scm_symbol_value_bool("bridge-show-input-state?")) {
+	!uim_scm_symbol_value_bool("bridge-show-input-state?")) {
 	Canddisp *disp = canddisp_singleton();
 	disp->hide_caret_state();
     }
@@ -173,11 +178,11 @@ void XimServer::customContext(const char *custom, const char *val) {
 }
 
 void XimServer::reloadConfigs() {
-#if 0
-    // uim_prop_reload_configs() seems broken... --ekato Aug 19 2005
+#if 1
     uim_prop_reload_configs();
-#else
     reload_uim(0);
+#else
+    reload_uim(1);
 #endif
 
     // Updated global IM of XimServer
@@ -400,7 +405,7 @@ InputContext::createUimContext(const char *engine)
     }
 
     uim_context uc = uim_create_context((void *) this, "UTF-8",
-					NULL, real_im, uim_iconv,
+					NULL, real_im, NULL,
 					InputContext::commit_cb);
 
     if (uc) {
@@ -512,8 +517,12 @@ void InputContext::review_im(const char *engine)
 	free(mLocaleName);
 	mLocaleName = locale;
     } else {
-	if (!is_locale_included(engine_locales, client_locale))
-	    changeContext(prev_engine);
+	if (!is_locale_included(engine_locales, client_locale)) {
+	    clear();
+	    uim_switch_im(mUc, prev_engine);
+	    free(mEngineName);
+	    mEngineName = strdup(prev_engine);
+	}
     }
     free(prev_engine);
 }
@@ -1096,6 +1105,10 @@ void keyState::check_key(keyEventX *x)
     if (x->state & Mod5Mask)
 	mModifier |= (gMod5Mask & mPreModState);
 
+#if UIM_XIM_USE_JAPANESE_KANA_KEYBOARD_HACK
+    mKey = uim_x_kana_input_hack_translate_key(x->key_sym,
+					       (KeyCode)x->ev.xkey.keycode);
+#endif
     if (x->key_sym < 128 && x->key_sym >= 32)
 	mKey = x->key_sym;
     else if (x->key_sym >= XK_F1 && x->key_sym <= XK_F35)
@@ -1275,6 +1288,11 @@ void init_modifier_keys() {
     gMod3Mask = check_modifier(Mod3MaskSyms);
     gMod4Mask = check_modifier(Mod4MaskSyms);
     gMod5Mask = check_modifier(Mod5MaskSyms);
+
+#if UIM_XIM_USE_JAPANESE_KANA_KEYBOARD_HACK
+    // Init at here to sync with proper update timing although not a modifier.
+    uim_x_kana_input_hack_init(XimServer::gDpy);
+#endif
 }
 
 
